# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-05-17 14:49
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('example', '0002_taggeditem'),
    ]

    operations = [
        migrations.CreateModel(
            name='Company',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
            ],
        ),
        migrations.CreateModel(
            name='Project',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('topic', models.CharField(max_length=30)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AlterField(
            model_name='comment',
            name='entry',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='comments', to='example.Entry'),
        ),
        migrations.CreateModel(
            name='ArtProject',
            fields=[
                ('project_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='example.Project')),
                ('artist', models.CharField(max_length=30)),
            ],
            options={
                'abstract': False,
            },
            bases=('example.project',),
        ),
        migrations.CreateModel(
            name='ResearchProject',
            fields=[
                ('project_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='example.Project')),
                ('supervisor', models.CharField(max_length=30)),
            ],
            options={
                'abstract': False,
            },
            bases=('example.project',),
        ),
        migrations.AddField(
            model_name='project',
            name='polymorphic_ctype',
            field=models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='polymorphic_example.project_set+', to='contenttypes.ContentType'),
        ),
        migrations.AddField(
            model_name='company',
            name='current_project',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='companies', to='example.Project'),
        ),
        migrations.AddField(
            model_name='company',
            name='future_projects',
            field=models.ManyToManyField(to='example.Project'),
        ),
    ]



import pytest
from django.urls import reverse

pytestmark = pytest.mark.django_db


def test_sparse_fieldset_ordered_dict_error(multiple_entries, client):
    base_url = reverse('entry-list')
    querystring = '?fields[entries]=blog,headline'
    # RuntimeError: OrderedDict mutated during iteration
    response = client.get(base_url + querystring)
    assert response.status_code == 200  # succeed if we didn't fail due to the above RuntimeError



from unittest import mock

import pytest
from django.urls import reverse

pytestmark = pytest.mark.django_db


@mock.patch(
    'rest_framework_json_api.utils'
    '.get_default_included_resources_from_serializer',
    new=lambda s: [])
def test_pagination_with_single_entry(single_entry, client):

    expected = {
        "data": [
            {
                "type": "posts",
                "id": "1",
                "attributes":
                {
                    "headline": single_entry.headline,
                    "bodyText": single_entry.body_text,
                    "pubDate": None,
                    "modDate": None
                },
                "meta": {
                    "bodyFormat": "text"
                },
                "relationships":
                {
                    "blog": {
                        "data": {"type": "blogs", "id": "1"}
                    },
                    "blogHyperlinked": {
                        "links": {
                            "related": "http://testserver/entries/1/blog",
                            "self": "http://testserver/entries/1/relationships/blog_hyperlinked",
                        }
                    },
                    "authors": {
                        "meta": {"count": 1},
                        "data": [{"type": "authors", "id": "1"}]
                    },
                    "comments": {
                        "meta": {"count": 1},
                        "data": [{"type": "comments", "id": "1"}]
                    },
                    "commentsHyperlinked": {
                        "links": {
                            "related": "http://testserver/entries/1/comments",
                            "self": "http://testserver/entries/1/relationships/comments_hyperlinked"
                        }
                    },
                    "suggested": {
                        "data": [],
                        "links": {
                            "related": "http://testserver/entries/1/suggested/",
                            "self": "http://testserver/entries/1/relationships/suggested"
                        }
                    },
                    "suggestedHyperlinked": {
                        "links": {
                            "related": "http://testserver/entries/1/suggested/",
                            "self": "http://testserver/entries/1"
                                    "/relationships/suggested_hyperlinked"
                        }
                    },
                    "featuredHyperlinked": {
                        "links": {
                            "related": "http://testserver/entries/1/featured",
                            "self": "http://testserver/entries/1/relationships/featured_hyperlinked"
                        }
                    },
                    "tags": {
                        'meta': {'count': 1},
                        "data": [
                            {
                                "id": "1",
                                "type": "taggedItems"
                            }
                        ]
                    }
                }
            }],
        "links": {
            'first': 'http://testserver/entries?page%5Bnumber%5D=1',
            'last': 'http://testserver/entries?page%5Bnumber%5D=1',
            "next": None,
            "prev": None,
        },
        "meta":
        {
            "pagination":
            {
                "page": 1,
                "pages": 1,
                "count": 1
            }
        }
    }

    response = client.get(reverse("entry-list"))

    assert expected == response.json()



# -*- encoding: utf-8 -*-

import factory
from faker import Factory as FakerFactory

from example.models import (
    ArtProject,
    Author,
    AuthorBio,
    AuthorBioMetadata,
    AuthorType,
    Blog,
    Comment,
    Company,
    Entry,
    ProjectType,
    ResearchProject,
    TaggedItem
)

faker = FakerFactory.create()
faker.seed(983843)


class BlogFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Blog

    name = factory.LazyAttribute(lambda x: faker.name())


class AuthorTypeFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = AuthorType

    name = factory.LazyAttribute(lambda x: faker.name())


class AuthorFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Author

    name = factory.LazyAttribute(lambda x: faker.name())
    email = factory.LazyAttribute(lambda x: faker.email())

    bio = factory.RelatedFactory('example.factories.AuthorBioFactory', 'author')
    type = factory.SubFactory(AuthorTypeFactory)


class AuthorBioFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = AuthorBio

    author = factory.SubFactory(AuthorFactory)
    body = factory.LazyAttribute(lambda x: faker.text())

    metadata = factory.RelatedFactory('example.factories.AuthorBioMetadataFactory', 'bio')


class AuthorBioMetadataFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = AuthorBioMetadata

    bio = factory.SubFactory(AuthorBioFactory)
    body = factory.LazyAttribute(lambda x: faker.text())


class EntryFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Entry

    headline = factory.LazyAttribute(lambda x: faker.sentence(nb_words=4))
    body_text = factory.LazyAttribute(lambda x: faker.text())

    blog = factory.SubFactory(BlogFactory)

    @factory.post_generation
    def authors(self, create, extracted, **kwargs):
        if extracted:
            if isinstance(extracted, (list, tuple)):
                for author in extracted:
                    self.authors.add(author)
            else:
                self.authors.add(extracted)


class CommentFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Comment

    entry = factory.SubFactory(EntryFactory)
    body = factory.LazyAttribute(lambda x: faker.text())
    author = factory.SubFactory(AuthorFactory)


class TaggedItemFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = TaggedItem

    content_object = factory.SubFactory(EntryFactory)
    tag = factory.LazyAttribute(lambda x: faker.word())


class ProjectTypeFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ProjectType

    name = factory.LazyAttribute(lambda x: faker.name())


class ArtProjectFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ArtProject

    topic = factory.LazyAttribute(lambda x: faker.catch_phrase())
    artist = factory.LazyAttribute(lambda x: faker.name())
    project_type = factory.SubFactory(ProjectTypeFactory)


class ResearchProjectFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ResearchProject

    topic = factory.LazyAttribute(lambda x: faker.catch_phrase())
    supervisor = factory.LazyAttribute(lambda x: faker.name())
    project_type = factory.SubFactory(ProjectTypeFactory)


class CompanyFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Company

    name = factory.LazyAttribute(lambda x: faker.company())
    current_project = factory.SubFactory(ArtProjectFactory)

    @factory.post_generation
    def future_projects(self, create, extracted, **kwargs):
        if not create:  # pragma: no cover
            return
        if extracted:
            for project in extracted:
                self.future_projects.add(project)



import re

from django_filters.rest_framework import DjangoFilterBackend
from rest_framework.exceptions import ValidationError
from rest_framework.settings import api_settings

from rest_framework_json_api.utils import format_value


class DjangoFilterBackend(DjangoFilterBackend):
    """
    A Django-style ORM filter implementation, using `django-filter`.

    This is not part of the jsonapi standard per-se, other than the requirement
    to use the `filter` keyword: This is an optional implementation of style of
    filtering in which each filter is an ORM expression as implemented by
    DjangoFilterBackend and seems to be in alignment with an interpretation of
    http://jsonapi.org/recommendations/#filtering, including relationship
    chaining. It also returns a 400 error for invalid filters.

    Filters can be:

    - A resource field
      equality test:

      ``?filter[qty]=123``

    - Apply other
      https://docs.djangoproject.com/en/stable/ref/models/querysets/#field-lookups
      operators:

      ``?filter[name.icontains]=bar`` or ``?filter[name.isnull]=true...``

    - Membership in
      a list of values:

      ``?filter[name.in]=abc,123,zzz`` (name in ['abc','123','zzz'])

    - Filters can be combined
      for intersection (AND):

      ``?filter[qty]=123&filter[name.in]=abc,123,zzz&filter[...]``

    - A related resource path
      can be used:

      ``?filter[inventory.item.partNum]=123456`` (where `inventory.item` is the relationship path)

    If you are also using rest_framework.filters.SearchFilter you'll want to customize
    the name of the query parameter for searching to make sure it doesn't conflict
    with a field name defined in the filterset.
    The recommended value is: `search_param="filter[search]"` but just make sure it's
    `filter[<something>]` to comply with the jsonapi spec requirement to use the filter
    keyword. The default is "search" unless overriden but it's used here just to make sure
    we don't complain about it being an invalid filter.
    """
    search_param = api_settings.SEARCH_PARAM

    # Make this regex check for 'filter' as well as 'filter[...]'
    # See http://jsonapi.org/format/#document-member-names for allowed characters
    # and http://jsonapi.org/format/#document-member-names-reserved-characters for reserved
    # characters (for use in paths, lists or as delimiters).
    # regex `\w` matches [a-zA-Z0-9_].
    # TODO: U+0080 and above allowed but not recommended. Leave them out for now.e
    #       Also, ' ' (space) is allowed within a member name but not recommended.
    filter_regex = re.compile(r'^filter(?P<ldelim>\[?)(?P<assoc>[\w\.\-]*)(?P<rdelim>\]?$)')

    def _validate_filter(self, keys, filterset_class):
        """
        Check that all the filter[key] are valid.

        :param keys: list of FilterSet keys
        :param filterset_class: :py:class:`django_filters.rest_framework.FilterSet`
        :raises ValidationError: if key not in FilterSet keys or no FilterSet.
        """
        for k in keys:
            if ((not filterset_class) or (k not in filterset_class.base_filters)):
                raise ValidationError("invalid filter[{}]".format(k))

    def get_filterset(self, request, queryset, view):
        """
        Sometimes there's no `filterset_class` defined yet the client still
        requests a filter. Make sure they see an error too. This means
        we have to `get_filterset_kwargs()` even if there's no `filterset_class`.
        """
        # TODO: .base_filters vs. .filters attr (not always present)
        filterset_class = self.get_filterset_class(view, queryset)
        kwargs = self.get_filterset_kwargs(request, queryset, view)
        self._validate_filter(kwargs.pop('filter_keys'), filterset_class)
        if filterset_class is None:
            return None
        return filterset_class(**kwargs)

    def get_filterset_kwargs(self, request, queryset, view):
        """
        Turns filter[<field>]=<value> into <field>=<value> which is what
        DjangoFilterBackend expects

        :raises ValidationError: for bad filter syntax
        """
        filter_keys = []
        # rewrite filter[field] query params to make DjangoFilterBackend work.
        data = request.query_params.copy()
        for qp, val in request.query_params.lists():
            m = self.filter_regex.match(qp)
            if m and (not m.groupdict()['assoc'] or
                      m.groupdict()['ldelim'] != '[' or m.groupdict()['rdelim'] != ']'):
                raise ValidationError("invalid query parameter: {}".format(qp))
            if m and qp != self.search_param:
                if not all(val):
                    raise ValidationError("missing value for query parameter {}".format(qp))
                # convert jsonapi relationship path to Django ORM's __ notation
                key = m.groupdict()['assoc'].replace('.', '__')
                # undo JSON_API_FORMAT_FIELD_NAMES conversion:
                key = format_value(key, 'underscore')
                data.setlist(key, val)
                filter_keys.append(key)
                del data[qp]
        return {
            'data': data,
            'queryset': queryset,
            'request': request,
            'filter_keys': filter_keys,
        }



# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot


snapshots = Snapshot()

snapshots['test_first_level_attribute_error 1'] = {
    'errors': [
        {
            'code': 'required',
            'detail': 'This field is required.',
            'source': {
                'pointer': '/data/attributes/headline'
            },
            'status': '400'
        }
    ]
}

snapshots['test_first_level_custom_attribute_error 1'] = {
    'errors': [
        {
            'detail': 'Too short',
            'source': {
                'pointer': '/data/attributes/body-text'
            },
            'title': 'Too Short title'
        }
    ]
}

snapshots['test_second_level_array_error 1'] = {
    'errors': [
        {
            'code': 'required',
            'detail': 'This field is required.',
            'source': {
                'pointer': '/data/attributes/comments/0/body'
            },
            'status': '400'
        }
    ]
}

snapshots['test_second_level_dict_error 1'] = {
    'errors': [
        {
            'code': 'required',
            'detail': 'This field is required.',
            'source': {
                'pointer': '/data/attributes/comment/body'
            },
            'status': '400'
        }
    ]
}

snapshots['test_third_level_array_error 1'] = {
    'errors': [
        {
            'code': 'required',
            'detail': 'This field is required.',
            'source': {
                'pointer': '/data/attributes/comments/0/attachments/0/data'
            },
            'status': '400'
        }
    ]
}

snapshots['test_third_level_custom_array_error 1'] = {
    'errors': [
        {
            'code': 'invalid',
            'detail': 'Too short data',
            'source': {
                'pointer': '/data/attributes/comments/0/attachments/0/data'
            },
            'status': '400'
        }
    ]
}

snapshots['test_third_level_dict_error 1'] = {
    'errors': [
        {
            'code': 'required',
            'detail': 'This field is required.',
            'source': {
                'pointer': '/data/attributes/comments/0/attachment/data'
            },
            'status': '400'
        }
    ]
}

snapshots['test_many_third_level_dict_errors 1'] = {
    'errors': [
        {
            'code': 'required',
            'detail': 'This field is required.',
            'source': {
                'pointer': '/data/attributes/comments/0/attachment/data'
            },
            'status': '400'
        },
        {
            'code': 'required',
            'detail': 'This field is required.',
            'source': {
                'pointer': '/data/attributes/comments/0/body'
            },
            'status': '400'
        }
    ]
}

snapshots['test_deprecation_warning 1'] = 'Rendering nested serializer as relationship is deprecated. Use `ResourceRelatedField` instead if DummyNestedSerializer in serializer example.tests.test_errors.test_deprecation_warning.<locals>.DummySerializer should remain a relationship. Otherwise set JSON_API_SERIALIZE_NESTED_SERIALIZERS_AS_ATTRIBUTE to True to render nested serializer as nested json attribute'



import rest_framework.exceptions as exceptions
import rest_framework.parsers
import rest_framework.renderers
from django_filters import rest_framework as filters
from rest_framework.filters import SearchFilter

import rest_framework_json_api.metadata
import rest_framework_json_api.parsers
import rest_framework_json_api.renderers
from rest_framework_json_api.django_filters import DjangoFilterBackend
from rest_framework_json_api.filters import OrderingFilter, QueryParameterValidationFilter
from rest_framework_json_api.pagination import JsonApiPageNumberPagination
from rest_framework_json_api.utils import format_drf_errors
from rest_framework_json_api.views import ModelViewSet, RelationshipView

from example.models import Author, Blog, Comment, Company, Entry, Project, ProjectType
from example.serializers import (
    AuthorSerializer,
    BlogDRFSerializer,
    BlogSerializer,
    CommentSerializer,
    CompanySerializer,
    EntryDRFSerializers,
    EntrySerializer,
    ProjectSerializer,
    ProjectTypeSerializer
)

HTTP_422_UNPROCESSABLE_ENTITY = 422


class BlogViewSet(ModelViewSet):
    queryset = Blog.objects.all()
    serializer_class = BlogSerializer

    def get_object(self):
        entry_pk = self.kwargs.get('entry_pk', None)
        if entry_pk is not None:
            return Entry.objects.get(id=entry_pk).blog

        return super(BlogViewSet, self).get_object()


class DRFBlogViewSet(ModelViewSet):
    queryset = Blog.objects.all()
    serializer_class = BlogDRFSerializer
    lookup_url_kwarg = 'entry_pk'

    def get_object(self):
        entry_pk = self.kwargs.get(self.lookup_url_kwarg, None)
        if entry_pk is not None:
            return Entry.objects.get(id=entry_pk).blog

        return super(DRFBlogViewSet, self).get_object()


class JsonApiViewSet(ModelViewSet):
    """
    This is an example on how to configure DRF-jsonapi from
    within a class. It allows using DRF-jsonapi alongside
    vanilla DRF API views.
    """
    parser_classes = [
        rest_framework_json_api.parsers.JSONParser,
        rest_framework.parsers.FormParser,
        rest_framework.parsers.MultiPartParser,
    ]
    renderer_classes = [
        rest_framework_json_api.renderers.JSONRenderer,
        rest_framework.renderers.BrowsableAPIRenderer,
    ]
    metadata_class = rest_framework_json_api.metadata.JSONAPIMetadata

    def handle_exception(self, exc):
        if isinstance(exc, exceptions.ValidationError):
            # some require that validation errors return 422 status
            # for example ember-data (isInvalid method on adapter)
            exc.status_code = HTTP_422_UNPROCESSABLE_ENTITY
        # exception handler can't be set on class so you have to
        # override the error response in this method
        response = super(JsonApiViewSet, self).handle_exception(exc)
        context = self.get_exception_handler_context()
        return format_drf_errors(response, context, exc)


class BlogCustomViewSet(JsonApiViewSet):
    queryset = Blog.objects.all()
    serializer_class = BlogSerializer


class EntryViewSet(ModelViewSet):
    queryset = Entry.objects.all()
    resource_name = 'posts'

    def get_serializer_class(self):
        return EntrySerializer

    def get_object(self):
        # Handle featured
        entry_pk = self.kwargs.get('entry_pk', None)
        if entry_pk is not None:
            return Entry.objects.exclude(pk=entry_pk).first()

        return super(EntryViewSet, self).get_object()


class DRFEntryViewSet(ModelViewSet):
    queryset = Entry.objects.all()
    serializer_class = EntryDRFSerializers
    lookup_url_kwarg = 'entry_pk'

    def get_object(self):
        # Handle featured
        entry_pk = self.kwargs.get(self.lookup_url_kwarg, None)
        if entry_pk is not None:
            return Entry.objects.exclude(pk=entry_pk).first()

        return super(DRFEntryViewSet, self).get_object()


class NoPagination(JsonApiPageNumberPagination):
    page_size = None


class NonPaginatedEntryViewSet(EntryViewSet):
    pagination_class = NoPagination
    # override the default filter backends in order to test QueryParameterValidationFilter without
    # breaking older usage of non-standard query params like `page_size`.
    filter_backends = (QueryParameterValidationFilter, OrderingFilter,
                       DjangoFilterBackend, SearchFilter)
    ordering_fields = ('headline', 'body_text', 'blog__name', 'blog__id')
    rels = ('exact', 'iexact',
            'contains', 'icontains',
            'gt', 'gte', 'lt', 'lte',
            'in', 'regex', 'isnull',)
    filterset_fields = {
        'id': ('exact', 'in'),
        'headline': rels,
        'body_text': rels,
        'blog__name': rels,
        'blog__tagline': rels,
    }
    search_fields = ('headline', 'body_text', 'blog__name', 'blog__tagline')


class EntryFilter(filters.FilterSet):
    bname = filters.CharFilter(field_name="blog__name",
                               lookup_expr="exact")

    authors__id = filters.ModelMultipleChoiceFilter(
        field_name='authors',
        to_field_name='id',
        conjoined=True,  # to "and" the ids
        queryset=Author.objects.all(),
    )

    class Meta:
        model = Entry
        fields = {
            'id': ('exact',),
            'headline': ('exact',),
            'body_text': ('exact',),
            'authors__id': ('in',),
        }


class FiltersetEntryViewSet(EntryViewSet):
    """
    like above but use filterset_class instead of filterset_fields
    """
    pagination_class = NoPagination
    filterset_fields = None
    filterset_class = EntryFilter
    filter_backends = (QueryParameterValidationFilter, DjangoFilterBackend,)


class NoFiltersetEntryViewSet(EntryViewSet):
    """
    like above but no filtersets
    """
    pagination_class = NoPagination
    filterset_fields = None
    filterset_class = None


class AuthorViewSet(ModelViewSet):
    queryset = Author.objects.all()
    serializer_class = AuthorSerializer


class CommentViewSet(ModelViewSet):
    queryset = Comment.objects.all()
    serializer_class = CommentSerializer
    select_for_includes = {
        'writer': ['author__bio']
    }
    prefetch_for_includes = {
        '__all__': [],
        'author': ['author__bio', 'author__entries'],
    }

    def get_queryset(self, *args, **kwargs):
        entry_pk = self.kwargs.get('entry_pk', None)
        if entry_pk is not None:
            return self.queryset.filter(entry_id=entry_pk)

        return super(CommentViewSet, self).get_queryset()


class CompanyViewset(ModelViewSet):
    queryset = Company.objects.all()
    serializer_class = CompanySerializer


class ProjectViewset(ModelViewSet):
    queryset = Project.objects.all().order_by('pk')
    serializer_class = ProjectSerializer


class ProjectTypeViewset(ModelViewSet):
    queryset = ProjectType.objects.all()
    serializer_class = ProjectTypeSerializer


class EntryRelationshipView(RelationshipView):
    queryset = Entry.objects.all()


class BlogRelationshipView(RelationshipView):
    queryset = Blog.objects.all()


class CommentRelationshipView(RelationshipView):
    queryset = Comment.objects.all()


class AuthorRelationshipView(RelationshipView):
    queryset = Author.objects.all()
    self_link_view_name = 'author-relationships'




from django.contrib.auth import get_user_model
from rest_framework.test import APITestCase


class TestBase(APITestCase):
    """
    Test base class to setup a couple users.
    """

    def setUp(self):
        """
        Create those users
        """
        super(TestBase, self).setUp()
        self.create_users()

    def create_user(self, username, email, password="pw",
                    first_name='', last_name=''):
        """
        Helper method to create a user
        """
        User = get_user_model()
        user = User.objects.create_user(
            username, email, password=password
        )
        if first_name or last_name:
            user.first_name = first_name
            user.last_name = last_name
            user.save()
        return user

    def create_users(self):
        """
        Create a couple users
        """
        self.john = self.create_user(
            'trane', 'john@example.com',
            first_name='John', last_name="Coltrane")
        self.miles = self.create_user(
            'miles', 'miles@example.com',
            first_name="Miles", last_name="Davis")



# Generated by Django 2.1.4 on 2018-12-28 07:52

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('example', '0005_auto_20180922_1508'),
    ]

    operations = [
        migrations.CreateModel(
            name='AuthorBioMetadata',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('modified_at', models.DateTimeField(auto_now=True)),
                ('body', models.TextField()),
                ('bio', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='metadata', to='example.AuthorBio')),
            ],
            options={
                'ordering': ('id',),
            },
        ),
        migrations.AlterField(
            model_name='comment',
            name='author',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='comments', to='example.Author'),
        ),
    ]



from django.utils.translation import gettext_lazy as _
from rest_framework import exceptions, status

from rest_framework_json_api import utils

from .settings import json_api_settings


def rendered_with_json_api(view):
    from rest_framework_json_api.renderers import JSONRenderer
    for renderer_class in getattr(view, 'renderer_classes', []):
        if issubclass(renderer_class, JSONRenderer):
            return True
    return False


def exception_handler(exc, context):
    # Import this here to avoid potential edge-case circular imports, which
    # crashes with:
    # "ImportError: Could not import 'rest_framework_json_api.parsers.JSONParser' for API setting
    # 'DEFAULT_PARSER_CLASSES'. ImportError: cannot import name 'exceptions'.'"
    #
    # Also see: https://github.com/django-json-api/django-rest-framework-json-api/issues/158
    from rest_framework.views import exception_handler as drf_exception_handler

    # Render exception with DRF
    response = drf_exception_handler(exc, context)
    if not response:
        return response

    # Use regular DRF format if not rendered by DRF JSON API and not uniform
    is_json_api_view = rendered_with_json_api(context['view'])
    is_uniform = json_api_settings.UNIFORM_EXCEPTIONS
    if not is_json_api_view and not is_uniform:
        return response

    # Convert to DRF JSON API error format
    response = utils.format_drf_errors(response, context, exc)

    # Add top-level 'errors' object when not rendered by DRF JSON API
    if not is_json_api_view:
        response.data = utils.format_errors(response.data)

    return response


class Conflict(exceptions.APIException):
    status_code = status.HTTP_409_CONFLICT
    default_detail = _('Conflict.')



import copy
import inspect
import operator
from collections import OrderedDict

import inflection
from django.conf import settings
from django.db.models import Manager
from django.db.models.fields.related_descriptors import (
    ManyToManyDescriptor,
    ReverseManyToOneDescriptor
)
from django.http import Http404
from django.utils import encoding
from django.utils.module_loading import import_string as import_class_from_dotted_path
from django.utils.translation import gettext_lazy as _
from rest_framework import exceptions
from rest_framework.exceptions import APIException

from .settings import json_api_settings

# Generic relation descriptor from django.contrib.contenttypes.
if 'django.contrib.contenttypes' not in settings.INSTALLED_APPS:  # pragma: no cover
    # Target application does not use contenttypes. Importing would cause errors.
    ReverseGenericManyToOneDescriptor = object()
else:
    from django.contrib.contenttypes.fields import ReverseGenericManyToOneDescriptor


def get_resource_name(context, expand_polymorphic_types=False):
    """
    Return the name of a resource.
    """
    from rest_framework_json_api.serializers import PolymorphicModelSerializer
    view = context.get('view')

    # Sanity check to make sure we have a view.
    if not view:
        return None

    # Check to see if there is a status code and return early
    # with the resource_name value of `errors`.
    try:
        code = str(view.response.status_code)
    except (AttributeError, ValueError):
        pass
    else:
        if code.startswith('4') or code.startswith('5'):
            return 'errors'

    try:
        resource_name = getattr(view, 'resource_name')
    except AttributeError:
        try:
            serializer = view.get_serializer_class()
            if expand_polymorphic_types and issubclass(serializer, PolymorphicModelSerializer):
                return serializer.get_polymorphic_types()
            else:
                return get_resource_type_from_serializer(serializer)
        except AttributeError:
            try:
                resource_name = get_resource_type_from_model(view.model)
            except AttributeError:
                resource_name = view.__class__.__name__

            if not isinstance(resource_name, str):
                # The resource name is not a string - return as is
                return resource_name

            # the name was calculated automatically from the view > pluralize and format
            resource_name = format_resource_type(resource_name)

    return resource_name


def get_serializer_fields(serializer):
    fields = None
    if hasattr(serializer, 'child'):
        fields = getattr(serializer.child, 'fields')
        meta = getattr(serializer.child, 'Meta', None)
    if hasattr(serializer, 'fields'):
        fields = getattr(serializer, 'fields')
        meta = getattr(serializer, 'Meta', None)

    if fields is not None:
        meta_fields = getattr(meta, 'meta_fields', {})
        for field in meta_fields:
            try:
                fields.pop(field)
            except KeyError:
                pass
        return fields


def format_field_names(obj, format_type=None):
    """
    Takes a dict and returns it with formatted keys as set in `format_type`
    or `JSON_API_FORMAT_FIELD_NAMES`

    :format_type: Either 'dasherize', 'camelize', 'capitalize' or 'underscore'
    """
    if format_type is None:
        format_type = json_api_settings.FORMAT_FIELD_NAMES

    if isinstance(obj, dict):
        formatted = OrderedDict()
        for key, value in obj.items():
            key = format_value(key, format_type)
            formatted[key] = value
        return formatted

    return obj


def format_value(value, format_type=None):
    if format_type is None:
        format_type = json_api_settings.FORMAT_FIELD_NAMES
    if format_type == 'dasherize':
        # inflection can't dasherize camelCase
        value = inflection.underscore(value)
        value = inflection.dasherize(value)
    elif format_type == 'camelize':
        value = inflection.camelize(value, False)
    elif format_type == 'capitalize':
        value = inflection.camelize(value)
    elif format_type == 'underscore':
        value = inflection.underscore(value)
    return value


def format_resource_type(value, format_type=None, pluralize=None):
    if format_type is None:
        format_type = json_api_settings.FORMAT_TYPES

    if pluralize is None:
        pluralize = json_api_settings.PLURALIZE_TYPES

    if format_type:
        # format_type will never be None here so we can use format_value
        value = format_value(value, format_type)

    return inflection.pluralize(value) if pluralize else value


def get_related_resource_type(relation):
    from rest_framework_json_api.serializers import PolymorphicModelSerializer
    try:
        return get_resource_type_from_serializer(relation)
    except AttributeError:
        pass
    relation_model = None
    if hasattr(relation, '_meta'):
        relation_model = relation._meta.model
    elif hasattr(relation, 'model'):
        # the model type was explicitly passed as a kwarg to ResourceRelatedField
        relation_model = relation.model
    elif hasattr(relation, 'get_queryset') and relation.get_queryset() is not None:
        relation_model = relation.get_queryset().model
    elif (
            getattr(relation, 'many', False) and
            hasattr(relation.child, 'Meta') and
            hasattr(relation.child.Meta, 'model')):
        # For ManyToMany relationships, get the model from the child
        # serializer of the list serializer
        relation_model = relation.child.Meta.model
    else:
        parent_serializer = relation.parent
        parent_model = None
        if isinstance(parent_serializer, PolymorphicModelSerializer):
            parent_model = parent_serializer.get_polymorphic_serializer_for_instance(
                parent_serializer.instance).Meta.model
        elif hasattr(parent_serializer, 'Meta'):
            parent_model = getattr(parent_serializer.Meta, 'model', None)
        elif hasattr(parent_serializer, 'parent') and hasattr(parent_serializer.parent, 'Meta'):
            parent_model = getattr(parent_serializer.parent.Meta, 'model', None)

        if parent_model is not None:
            if relation.source:
                if relation.source != '*':
                    parent_model_relation = getattr(parent_model, relation.source)
                else:
                    parent_model_relation = getattr(parent_model, relation.field_name)
            else:
                parent_model_relation = getattr(parent_model, parent_serializer.field_name)

            parent_model_relation_type = type(parent_model_relation)
            if parent_model_relation_type is ReverseManyToOneDescriptor:
                relation_model = parent_model_relation.rel.related_model
            elif parent_model_relation_type is ManyToManyDescriptor:
                relation_model = parent_model_relation.field.remote_field.model
                # In case we are in a reverse relation
                if relation_model == parent_model:
                    relation_model = parent_model_relation.field.model
            elif parent_model_relation_type is ReverseGenericManyToOneDescriptor:
                relation_model = parent_model_relation.rel.model
            elif hasattr(parent_model_relation, 'field'):
                try:
                    relation_model = parent_model_relation.field.remote_field.model
                except AttributeError:
                    relation_model = parent_model_relation.field.related.model
            else:
                return get_related_resource_type(parent_model_relation)

    if relation_model is None:
        raise APIException(_('Could not resolve resource type for relation %s' % relation))

    return get_resource_type_from_model(relation_model)


def get_resource_type_from_model(model):
    json_api_meta = getattr(model, 'JSONAPIMeta', None)
    return getattr(
        json_api_meta,
        'resource_name',
        format_resource_type(model.__name__))


def get_resource_type_from_queryset(qs):
    return get_resource_type_from_model(qs.model)


def get_resource_type_from_instance(instance):
    if hasattr(instance, '_meta'):
        return get_resource_type_from_model(instance._meta.model)


def get_resource_type_from_manager(manager):
    return get_resource_type_from_model(manager.model)


def get_resource_type_from_serializer(serializer):
    json_api_meta = getattr(serializer, 'JSONAPIMeta', None)
    meta = getattr(serializer, 'Meta', None)
    if hasattr(json_api_meta, 'resource_name'):
        return json_api_meta.resource_name
    elif hasattr(meta, 'resource_name'):
        return meta.resource_name
    elif hasattr(meta, 'model'):
        return get_resource_type_from_model(meta.model)
    raise AttributeError()


def get_included_resources(request, serializer=None):
    """ Build a list of included resources. """
    include_resources_param = request.query_params.get('include') if request else None
    if include_resources_param:
        return include_resources_param.split(',')
    else:
        return get_default_included_resources_from_serializer(serializer)


def get_default_included_resources_from_serializer(serializer):
    meta = getattr(serializer, 'JSONAPIMeta', None)
    if meta is None and getattr(serializer, 'many', False):
        meta = getattr(serializer.child, 'JSONAPIMeta', None)
    return list(getattr(meta, 'included_resources', []))


def get_included_serializers(serializer):
    included_serializers = copy.copy(getattr(serializer, 'included_serializers', dict()))

    for name, value in iter(included_serializers.items()):
        if not isinstance(value, type):
            if value == 'self':
                included_serializers[name] = (
                    serializer if isinstance(serializer, type) else serializer.__class__
                )
            else:
                included_serializers[name] = import_class_from_dotted_path(value)

    return included_serializers


def get_relation_instance(resource_instance, source, serializer):
    try:
        relation_instance = operator.attrgetter(source)(resource_instance)
    except AttributeError:
        # if the field is not defined on the model then we check the serializer
        # and if no value is there we skip over the field completely
        serializer_method = getattr(serializer, source, None)
        if serializer_method and hasattr(serializer_method, '__call__'):
            relation_instance = serializer_method(resource_instance)
        else:
            return False, None

    if isinstance(relation_instance, Manager):
        relation_instance = relation_instance.all()

    return True, relation_instance


class Hyperlink(str):
    """
    A string like object that additionally has an associated name.
    We use this for hyperlinked URLs that may render as a named link
    in some contexts, or render as a plain URL in others.

    Comes from Django REST framework 3.2
    https://github.com/tomchristie/django-rest-framework
    """

    def __new__(self, url, name):
        ret = str.__new__(self, url)
        ret.name = name
        return ret

    is_hyperlink = True


def format_drf_errors(response, context, exc):
    errors = []
    # handle generic errors. ValidationError('test') in a view for example
    if isinstance(response.data, list):
        for message in response.data:
            errors.extend(format_error_object(message, '/data', response))
    # handle all errors thrown from serializers
    else:
        for field, error in response.data.items():
            field = format_value(field)
            pointer = '/data/attributes/{}'.format(field)
            if isinstance(exc, Http404) and isinstance(error, str):
                # 404 errors don't have a pointer
                errors.extend(format_error_object(error, None, response))
            elif isinstance(error, str):
                classes = inspect.getmembers(exceptions, inspect.isclass)
                # DRF sets the `field` to 'detail' for its own exceptions
                if isinstance(exc, tuple(x[1] for x in classes)):
                    pointer = '/data'
                errors.extend(format_error_object(error, pointer, response))
            elif isinstance(error, list):
                errors.extend(format_error_object(error, pointer, response))
            else:
                errors.extend(format_error_object(error, pointer, response))

    context['view'].resource_name = 'errors'
    response.data = errors

    return response


def format_error_object(message, pointer, response):
    errors = []
    if isinstance(message, dict):

        # as there is no required field in error object we check that all fields are string
        # except links and source which might be a dict
        is_custom_error = all([
            isinstance(value, str)
            for key, value in message.items() if key not in ['links', 'source']
        ])

        if is_custom_error:
            if 'source' not in message:
                message['source'] = {}
            message['source'] = {
                'pointer': pointer,
            }
            errors.append(message)
        else:
            for k, v in message.items():
                errors.extend(format_error_object(v, pointer + '/{}'.format(k), response))
    elif isinstance(message, list):
        for num, error in enumerate(message):
            if isinstance(error, (list, dict)):
                new_pointer = pointer + '/{}'.format(num)
            else:
                new_pointer = pointer
            if error:
                errors.extend(format_error_object(error, new_pointer, response))
    else:
        error_obj = {
            'detail': message,
            'status': encoding.force_str(response.status_code),
        }
        if pointer is not None:
            error_obj['source'] = {
                'pointer': pointer,
            }
        code = getattr(message, "code", None)
        if code is not None:
            error_obj['code'] = code
        errors.append(error_obj)

    return errors


def format_errors(data):
    if len(data) > 1 and isinstance(data, list):
        data.sort(key=lambda x: x.get('source', {}).get('pointer', ''))
    return {'errors': data}



# -*- encoding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib.contenttypes.fields import GenericForeignKey, GenericRelation
from django.contrib.contenttypes.models import ContentType
from django.db import models
from polymorphic.models import PolymorphicModel


class BaseModel(models.Model):
    """
    I hear RoR has this by default, who doesn't need these two fields!
    """
    created_at = models.DateTimeField(auto_now_add=True)
    modified_at = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True


class TaggedItem(BaseModel):
    tag = models.SlugField()
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')

    def __str__(self):
        return self.tag

    class Meta:
        ordering = ('id',)


class Blog(BaseModel):
    name = models.CharField(max_length=100)
    tagline = models.TextField()
    tags = GenericRelation(TaggedItem)

    def __str__(self):
        return self.name

    class Meta:
        ordering = ('id',)


class AuthorType(BaseModel):
    name = models.CharField(max_length=50)

    def __str__(self):
        return self.name

    class Meta:
        ordering = ('id',)


class Author(BaseModel):
    name = models.CharField(max_length=50)
    email = models.EmailField()
    type = models.ForeignKey(AuthorType, null=True, on_delete=models.CASCADE)

    def __str__(self):
        return self.name

    class Meta:
        ordering = ('id',)


class AuthorBio(BaseModel):
    author = models.OneToOneField(Author, related_name='bio', on_delete=models.CASCADE)
    body = models.TextField()

    def __str__(self):
        return self.author.name

    class Meta:
        ordering = ('id',)


class AuthorBioMetadata(BaseModel):
    """
    Just a class to have a relation with author bio
    """
    bio = models.OneToOneField(AuthorBio, related_name='metadata', on_delete=models.CASCADE)
    body = models.TextField()

    def __str__(self):
        return self.bio.author.name

    class Meta:
        ordering = ('id',)


class Entry(BaseModel):
    blog = models.ForeignKey(Blog, on_delete=models.CASCADE)
    headline = models.CharField(max_length=255)
    body_text = models.TextField(null=True)
    pub_date = models.DateField(null=True)
    mod_date = models.DateField(null=True)
    authors = models.ManyToManyField(Author, related_name='entries')
    n_comments = models.IntegerField(default=0)
    n_pingbacks = models.IntegerField(default=0)
    rating = models.IntegerField(default=0)
    tags = GenericRelation(TaggedItem)

    def __str__(self):
        return self.headline

    class Meta:
        ordering = ('id',)


class Comment(BaseModel):
    entry = models.ForeignKey(Entry, related_name='comments', on_delete=models.CASCADE)
    body = models.TextField()
    author = models.ForeignKey(
        Author,
        null=True,
        blank=True,
        on_delete=models.CASCADE,
        related_name='comments',
    )

    def __str__(self):
        return self.body

    class Meta:
        ordering = ('id',)


class ProjectType(BaseModel):
    name = models.CharField(max_length=50)

    def __str__(self):
        return self.name

    class Meta:
        ordering = ('id',)


class Project(PolymorphicModel):
    topic = models.CharField(max_length=30)
    project_type = models.ForeignKey(ProjectType, null=True, on_delete=models.CASCADE)


class ArtProject(Project):
    artist = models.CharField(max_length=30)
    description = models.CharField(max_length=100, null=True)


class ResearchProject(Project):
    supervisor = models.CharField(max_length=30)


class LabResults(models.Model):
    research_project = models.ForeignKey(
        ResearchProject, related_name='lab_results', on_delete=models.CASCADE)
    date = models.DateField()
    measurements = models.TextField()


class Company(models.Model):
    name = models.CharField(max_length=100)
    current_project = models.ForeignKey(
        Project, related_name='companies', on_delete=models.CASCADE)
    future_projects = models.ManyToManyField(Project)

    def __str__(self):
        return self.name



"""
Test rest_framework_json_api's utils functions.
"""
from rest_framework_json_api import utils

from example.serializers import AuthorSerializer, EntrySerializer
from example.tests import TestBase


class GetRelatedResourceTests(TestBase):
    """
    Ensure the `get_related_resource_type` function returns correct types.
    """

    def test_reverse_relation(self):
        """
        Ensure reverse foreign keys have their types identified correctly.
        """
        serializer = EntrySerializer()
        field = serializer.fields['comments']

        self.assertEqual(utils.get_related_resource_type(field), 'comments')

    def test_m2m_relation(self):
        """
        Ensure m2ms have their types identified correctly.
        """
        serializer = EntrySerializer()
        field = serializer.fields['authors']

        self.assertEqual(utils.get_related_resource_type(field), 'authors')

    def test_m2m_reverse_relation(self):
        """
        Ensure reverse m2ms have their types identified correctly.
        """
        serializer = AuthorSerializer()
        field = serializer.fields['entries']

        self.assertEqual(utils.get_related_resource_type(field), 'entries')



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-02 08:26
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Author',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('modified_at', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=50)),
                ('email', models.EmailField(max_length=254)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AuthorBio',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('modified_at', models.DateTimeField(auto_now=True)),
                ('body', models.TextField()),
                ('author', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='bio', to='example.Author')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Blog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('modified_at', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=100)),
                ('tagline', models.TextField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Comment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('modified_at', models.DateTimeField(auto_now=True)),
                ('body', models.TextField()),
                ('author', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='example.Author')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Entry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('modified_at', models.DateTimeField(auto_now=True)),
                ('headline', models.CharField(max_length=255)),
                ('body_text', models.TextField(null=True)),
                ('pub_date', models.DateField(null=True)),
                ('mod_date', models.DateField(null=True)),
                ('n_comments', models.IntegerField(default=0)),
                ('n_pingbacks', models.IntegerField(default=0)),
                ('rating', models.IntegerField(default=0)),
                ('authors', models.ManyToManyField(to='example.Author')),
                ('blog', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='example.Blog')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='comment',
            name='entry',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='example.Entry'),
        ),
    ]



import pytest
from pytest_factoryboy import register
from rest_framework.test import APIClient

from example.factories import (
    ArtProjectFactory,
    AuthorBioFactory,
    AuthorBioMetadataFactory,
    AuthorFactory,
    AuthorTypeFactory,
    BlogFactory,
    CommentFactory,
    CompanyFactory,
    EntryFactory,
    ResearchProjectFactory,
    TaggedItemFactory
)

register(BlogFactory)
register(AuthorFactory)
register(AuthorBioFactory)
register(AuthorBioMetadataFactory)
register(AuthorTypeFactory)
register(EntryFactory)
register(CommentFactory)
register(TaggedItemFactory)
register(ArtProjectFactory)
register(ResearchProjectFactory)
register(CompanyFactory)


@pytest.fixture
def single_entry(blog, author, entry_factory, comment_factory, tagged_item_factory):

    entry = entry_factory(blog=blog, authors=(author,))
    comment_factory(entry=entry)
    tagged_item_factory(content_object=entry)
    return entry


@pytest.fixture
def multiple_entries(blog_factory, author_factory, entry_factory, comment_factory):

    entries = [
        entry_factory(blog=blog_factory(), authors=(author_factory(),)),
        entry_factory(blog=blog_factory(), authors=(author_factory(),)),
    ]
    comment_factory(entry=entries[0])
    comment_factory(entry=entries[1])
    return entries


@pytest.fixture
def single_comment(blog, author, entry_factory, comment_factory):
    entry = entry_factory(blog=blog, authors=(author,))
    comment_factory(entry=entry)
    return comment_factory(entry=entry)


@pytest.fixture
def single_company(art_project_factory, research_project_factory, company_factory):
    company = company_factory(future_projects=(research_project_factory(), art_project_factory()))
    return company


@pytest.fixture
def single_art_project(art_project_factory):
    return art_project_factory()


@pytest.fixture
def client():
    return APIClient()



import pytest
from django.contrib.auth import get_user_model

from rest_framework_json_api import serializers
from rest_framework_json_api.renderers import JSONRenderer

pytestmark = pytest.mark.django_db


class ResourceSerializer(serializers.ModelSerializer):
    version = serializers.SerializerMethodField()

    def get_version(self, obj):
        return '1.0.0'

    class Meta:
        fields = ('username',)
        meta_fields = ('version',)
        model = get_user_model()


def test_build_json_resource_obj():
    resource = {
        'pk': 1,
        'username': 'Alice',
    }

    serializer = ResourceSerializer(data={'username': 'Alice'})
    serializer.is_valid()
    resource_instance = serializer.save()

    output = {
        'type': 'user',
        'id': '1',
        'attributes': {
            'username': 'Alice'
        },
    }

    assert JSONRenderer.build_json_resource_obj(
        serializer.fields, resource, resource_instance, 'user') == output


def test_can_override_methods():
    """
    Make sure extract_attributes and extract_relationships can be overriden.
    """
    resource = {
        'pk': 1,
        'username': 'Alice',
    }

    serializer = ResourceSerializer(data={'username': 'Alice'})
    serializer.is_valid()
    resource_instance = serializer.save()

    output = {
        'type': 'user',
        'id': '1',
        'attributes': {
            'username': 'Alice'
        },
    }

    class CustomRenderer(JSONRenderer):
        extract_attributes_was_overriden = False
        extract_relationships_was_overriden = False

        @classmethod
        def extract_attributes(cls, fields, resource):
            cls.extract_attributes_was_overriden = True
            return super(CustomRenderer, cls).extract_attributes(fields, resource)

        @classmethod
        def extract_relationships(cls, fields, resource, resource_instance):
            cls.extract_relationships_was_overriden = True
            return super(CustomRenderer, cls).extract_relationships(
                fields, resource, resource_instance
            )

    assert CustomRenderer.build_json_resource_obj(
        serializer.fields, resource, resource_instance, 'user') == output
    assert CustomRenderer.extract_attributes_was_overriden
    assert CustomRenderer.extract_relationships_was_overriden


def test_extract_attributes():
    fields = {
        'id': serializers.Field(),
        'username': serializers.Field(),
        'deleted': serializers.ReadOnlyField(),
    }
    resource = {'id': 1, 'deleted': None, 'username': 'jerel'}
    expected = {
        'username': 'jerel',
        'deleted': None
    }
    assert sorted(JSONRenderer.extract_attributes(fields, resource)) == sorted(expected), (
        'Regular fields should be extracted'
    )
    assert sorted(JSONRenderer.extract_attributes(fields, {})) == sorted(
        {'username': ''}), 'Should not extract read_only fields on empty serializer'


def test_extract_meta():
    serializer = ResourceSerializer(data={'username': 'jerel', 'version': '1.0.0'})
    serializer.is_valid()
    expected = {
        'version': '1.0.0',
    }
    assert JSONRenderer.extract_meta(serializer, serializer.data) == expected


class ExtractRootMetaResourceSerializer(ResourceSerializer):
    def get_root_meta(self, resource, many):
        if many:
            return {
                'foo': 'meta-many-value'
            }
        else:
            return {
                'foo': 'meta-value'
            }


class InvalidExtractRootMetaResourceSerializer(ResourceSerializer):
    def get_root_meta(self, resource, many):
        return 'not a dict'


def test_extract_root_meta():
    serializer = ExtractRootMetaResourceSerializer()
    expected = {
        'foo': 'meta-value',
    }
    assert JSONRenderer.extract_root_meta(serializer, {}) == expected


def test_extract_root_meta_many():
    serializer = ExtractRootMetaResourceSerializer(many=True)
    expected = {
        'foo': 'meta-many-value'
    }
    assert JSONRenderer.extract_root_meta(serializer, {}) == expected


def test_extract_root_meta_invalid_meta():
    serializer = InvalidExtractRootMetaResourceSerializer()
    with pytest.raises(AssertionError):
        JSONRenderer.extract_root_meta(serializer, {})



import pytest

from rest_framework_json_api.settings import json_api_settings


def test_settings_invalid():
    with pytest.raises(AttributeError):
        json_api_settings.INVALID_SETTING


def test_settings_default():
    assert json_api_settings.UNIFORM_EXCEPTIONS is False


def test_settings_override(settings):
    settings.JSON_API_FORMAT_FIELD_NAMES = 'dasherize'
    assert json_api_settings.FORMAT_FIELD_NAMES == 'dasherize'



from django.contrib.auth import models as auth_models
from rest_framework import serializers


class IdentitySerializer(serializers.ModelSerializer):
    """
    Identity Serializer
    """

    def validate_first_name(self, data):
        if len(data) > 10:
            raise serializers.ValidationError(
                'There\'s a problem with first name')
        return data

    def validate_last_name(self, data):
        if len(data) > 10:
            raise serializers.ValidationError(
                {
                    'id': 'armageddon101',
                    'detail': 'Hey! You need a last name!',
                    'meta': 'something',
                }
            )
        return data

    class Meta:
        model = auth_models.User
        fields = (
            'id', 'first_name', 'last_name', 'email', )



from django.test import override_settings
from django.urls import reverse

from example.tests import TestBase


class GenericViewSet(TestBase):
    """
    Test expected responses coming from a Generic ViewSet
    """

    def test_default_rest_framework_behavior(self):
        """
        This is more of an example really, showing default behavior
        """
        url = reverse('user-default', kwargs={'pk': self.miles.pk})

        response = self.client.get(url)

        self.assertEqual(200, response.status_code)

        expected = {
            'id': 2,
            'first_name': 'Miles',
            'last_name': 'Davis',
            'email': 'miles@example.com'
        }

        assert expected == response.json()

    def test_ember_expected_renderer(self):
        """
        The :class:`UserEmber` ViewSet has the ``resource_name`` of 'data'
        so that should be the key in the JSON response.
        """
        url = reverse('user-manual-resource-name', kwargs={'pk': self.miles.pk})

        with override_settings(JSON_API_FORMAT_FIELD_NAMES='dasherize'):
            response = self.client.get(url)
        self.assertEqual(200, response.status_code)

        expected = {
            'data': {
                'type': 'data',
                'id': '2',
                'attributes': {
                    'first-name': 'Miles',
                    'last-name': 'Davis',
                    'email': 'miles@example.com'
                }
            }
        }

        assert expected == response.json()

    def test_default_validation_exceptions(self):
        """
        Default validation exceptions should conform to json api spec
        """
        expected = {
            'errors': [
                {
                    'status': '400',
                    'source': {
                        'pointer': '/data/attributes/email',
                    },
                    'detail': 'Enter a valid email address.',
                    'code': 'invalid',
                },
                {
                    'status': '400',
                    'source': {
                        'pointer': '/data/attributes/first-name',
                    },
                    'detail': 'There\'s a problem with first name',
                    'code': 'invalid',
                }
            ]
        }
        with override_settings(JSON_API_FORMAT_FIELD_NAMES='dasherize'):
            response = self.client.post('/identities', {
                'data': {
                    'type': 'users',
                    'attributes': {
                        'email': 'bar', 'first_name': 'alajflajaljalajlfjafljalj'
                    }
                }
            })

        assert expected == response.json()

    def test_custom_validation_exceptions(self):
        """
        Exceptions should be able to be formatted manually
        """
        expected = {
            'errors': [
                {
                    'status': '400',
                    'source': {
                        'pointer': '/data/attributes/email',
                    },
                    'detail': 'Enter a valid email address.',
                    'code': 'invalid',
                },
                {
                    'id': 'armageddon101',
                    'detail': 'Hey! You need a last name!',
                    'meta': 'something',
                    'source': {'pointer': '/data/attributes/lastName'}
                },
            ]
        }
        response = self.client.post('/identities', {
            'data': {
                'type': 'users',
                'attributes': {
                    'email': 'bar', 'last_name': 'alajflajaljalajlfjafljalj'
                }
            }
        })

        assert expected == response.json()



from unittest import mock

import pytest
from django.test import TestCase
from django.urls import reverse
from django.utils import timezone
from rest_framework.request import Request
from rest_framework.test import APIRequestFactory

from rest_framework_json_api.serializers import (
    DateField,
    ModelSerializer,
    ResourceIdentifierObjectSerializer,
    empty
)
from rest_framework_json_api.utils import format_resource_type

from example.factories import ArtProjectFactory
from example.models import Author, Blog, Entry
from example.serializers import ArtProjectSerializer, BlogSerializer, ProjectSerializer

request_factory = APIRequestFactory()
pytestmark = pytest.mark.django_db


class TestResourceIdentifierObjectSerializer(TestCase):
    def setUp(self):
        self.blog = Blog.objects.create(name='Some Blog', tagline="It's a blog")
        now = timezone.now()

        self.entry = Entry.objects.create(
            blog=self.blog,
            headline='headline',
            body_text='body_text',
            pub_date=now.date(),
            mod_date=now.date(),
            n_comments=0,
            n_pingbacks=0,
            rating=3
        )
        for i in range(1, 6):
            name = 'some_author{}'.format(i)
            self.entry.authors.add(
                Author.objects.create(name=name, email='{}@example.org'.format(name))
            )

    def test_forward_relationship_not_loaded_when_not_included(self):
        to_representation_method = 'example.serializers.TaggedItemSerializer.to_representation'
        with mock.patch(to_representation_method) as mocked_serializer:
            class EntrySerializer(ModelSerializer):
                blog = BlogSerializer()

                class Meta:
                    model = Entry
                    fields = '__all__'

            request_without_includes = Request(request_factory.get('/'))
            serializer = EntrySerializer(context={'request': request_without_includes})
            serializer.to_representation(self.entry)

            mocked_serializer.assert_not_called()

    def test_forward_relationship_optimization_correct_representation(self):
        class EntrySerializer(ModelSerializer):
            blog = BlogSerializer()

            class Meta:
                model = Entry
                fields = '__all__'

        request_without_includes = Request(request_factory.get('/'))
        serializer = EntrySerializer(context={'request': request_without_includes})
        result = serializer.to_representation(self.entry)

        # Remove non deterministic fields
        result.pop('created_at')
        result.pop('modified_at')

        expected = dict(
            [
                ('id', 1),
                ('blog', dict([
                    ('name', 'Some Blog'),
                    ('tags', []),
                    ('copyright', 2020),
                    ('url', 'http://testserver/blogs/1')
                ])),
                ('headline', 'headline'),
                ('body_text', 'body_text'),
                ('pub_date', DateField().to_representation(self.entry.pub_date)),
                ('mod_date', DateField().to_representation(self.entry.mod_date)),
                ('n_comments', 0),
                ('n_pingbacks', 0),
                ('rating', 3),
                ('authors',
                    [
                        dict([('type', 'authors'), ('id', '1')]),
                        dict([('type', 'authors'), ('id', '2')]),
                        dict([('type', 'authors'), ('id', '3')]),
                        dict([('type', 'authors'), ('id', '4')]),
                        dict([('type', 'authors'), ('id', '5')])])])

        self.assertDictEqual(expected, result)

    def test_data_in_correct_format_when_instantiated_with_blog_object(self):
        serializer = ResourceIdentifierObjectSerializer(instance=self.blog)

        expected_data = {'type': format_resource_type('Blog'), 'id': str(self.blog.id)}

        assert serializer.data == expected_data

    def test_data_in_correct_format_when_instantiated_with_entry_object(self):
        serializer = ResourceIdentifierObjectSerializer(instance=self.entry)

        expected_data = {'type': format_resource_type('Entry'), 'id': str(self.entry.id)}

        assert serializer.data == expected_data

    def test_deserialize_primitive_data_blog(self):
        initial_data = {
            'type': format_resource_type('Blog'),
            'id': str(self.blog.id)
        }
        serializer = ResourceIdentifierObjectSerializer(data=initial_data, model_class=Blog)

        self.assertTrue(serializer.is_valid(), msg=serializer.errors)
        assert serializer.validated_data == self.blog

    def test_deserialize_primitive_data_blog_with_unexisting_pk(self):
        unexisting_pk = self.blog.id
        self.blog.delete()
        assert not Blog.objects.filter(id=unexisting_pk).exists()

        initial_data = {
            'type': format_resource_type('Blog'),
            'id': str(unexisting_pk)
        }
        serializer = ResourceIdentifierObjectSerializer(data=initial_data, model_class=Blog)

        self.assertFalse(serializer.is_valid())
        self.assertEqual(serializer.errors[0].code, 'does_not_exist')

    def test_data_in_correct_format_when_instantiated_with_queryset(self):
        qs = Author.objects.all()
        serializer = ResourceIdentifierObjectSerializer(instance=qs, many=True)

        type_string = format_resource_type('Author')
        author_pks = Author.objects.values_list('pk', flat=True)
        expected_data = [{'type': type_string, 'id': str(pk)} for pk in author_pks]

        assert serializer.data == expected_data

    def test_deserialize_many(self):
        type_string = format_resource_type('Author')
        author_pks = Author.objects.values_list('pk', flat=True)
        initial_data = [{'type': type_string, 'id': str(pk)} for pk in author_pks]

        serializer = ResourceIdentifierObjectSerializer(
            data=initial_data, model_class=Author, many=True
        )

        self.assertTrue(serializer.is_valid(), msg=serializer.errors)

        print(serializer.data)


class TestModelSerializer(object):
    def test_model_serializer_with_implicit_fields(self, comment, client):
        expected = {
            "data": {
                "type": "comments",
                "id": str(comment.pk),
                "attributes": {
                    "body": comment.body
                },
                "relationships": {
                    "entry": {
                        "data": {
                            "type": "entries",
                            "id": str(comment.entry.pk)
                        }
                    },
                    "author": {
                        "data": {
                            "type": "authors",
                            "id": str(comment.author.pk)
                        }
                    },
                    "writer": {
                        "data": {
                            "type": "writers",
                            "id": str(comment.author.pk)
                        }
                    },
                }
            }
        }

        response = client.get(reverse("comment-detail", kwargs={'pk': comment.pk}))

        assert response.status_code == 200
        assert expected == response.json()


class TestPolymorphicModelSerializer(TestCase):
    def setUp(self):
        self.project = ArtProjectFactory.create()
        self.child_init_args = {}

        # Override `__init__` with our own method
        def overridden_init(child_self, instance=None, data=empty, **kwargs):
            """
            Override `ArtProjectSerializer.__init__` with the same signature that
            `BaseSerializer.__init__` has to assert that it receives the parameters
            that `BaseSerializer` expects
            """
            self.child_init_args = dict(instance=instance, data=data, **kwargs)

            return super(ArtProjectSerializer, child_self).__init__(
                instance, data, **kwargs
            )

        self.child_serializer_init = ArtProjectSerializer.__init__
        ArtProjectSerializer.__init__ = overridden_init

    def tearDown(self):
        # Restore original init to avoid affecting other tests
        ArtProjectSerializer.__init__ = self.child_serializer_init

    def test_polymorphic_model_serializer_passes_instance_to_child(self):
        """
        Ensure that `PolymorphicModelSerializer` is passing the instance to the
        child serializer when initializing them
        """
        # Initialize a serializer that would partially update a model instance
        initial_data = {"artist": "Mark Bishop", "type": "artProjects"}
        parent_serializer = ProjectSerializer(
            instance=self.project, data=initial_data, partial=True
        )

        parent_serializer.is_valid(raise_exception=True)

        # Run save to force `ProjectSerializer` to init `ArtProjectSerializer`
        parent_serializer.save()

        # Assert that child init received the expected arguments
        assert self.child_init_args["instance"] == self.project
        assert self.child_init_args["data"] == initial_data
        assert self.child_init_args["partial"] == parent_serializer.partial
        assert self.child_init_args["context"] == parent_serializer.context



import json
from datetime import datetime

import pytest
from django.urls import reverse
from rest_framework import viewsets
from rest_framework.serializers import ModelSerializer, SerializerMethodField

from rest_framework_json_api.renderers import JSONRenderer

from example.models import Blog, Comment, Entry


# serializers
class RelatedModelSerializer(ModelSerializer):
    class Meta:
        model = Comment
        fields = ('id',)


class DummyTestSerializer(ModelSerializer):
    """
    This serializer is a simple compound document serializer which includes only
    a single embedded relation
    """
    related_models = RelatedModelSerializer(source='comments', many=True, read_only=True)

    json_field = SerializerMethodField()

    def get_json_field(self, entry):
        return {'JsonKey': 'JsonValue'}

    class Meta:
        model = Entry
        fields = ('related_models', 'json_field')


# views
class DummyTestViewSet(viewsets.ModelViewSet):
    queryset = Entry.objects.all()
    serializer_class = DummyTestSerializer


def render_dummy_test_serialized_view(view_class):
    serializer = DummyTestSerializer(instance=Entry())
    renderer = JSONRenderer()
    return renderer.render(
        serializer.data,
        renderer_context={'view': view_class()})


# tests
def test_simple_reverse_relation_included_renderer():
    """
    Test renderer when a single reverse fk relation is passed.
    """
    rendered = render_dummy_test_serialized_view(
        DummyTestViewSet)

    assert rendered


def test_render_format_field_names(settings):
    """Test that json field is kept untouched."""
    settings.JSON_API_FORMAT_FIELD_NAMES = 'dasherize'
    rendered = render_dummy_test_serialized_view(DummyTestViewSet)

    result = json.loads(rendered.decode())
    assert result['data']['attributes']['json-field'] == {'JsonKey': 'JsonValue'}


@pytest.mark.django_db
def test_blog_create(client):

    url = reverse('drf-entry-blog-list')
    name = "Dummy Name"

    request_data = {
        'data': {
            'attributes': {'name': name},
            'type': 'blogs'
        },
    }

    resp = client.post(url, request_data)

    # look for created blog in database
    blog = Blog.objects.filter(name=name)

    # check if blog exists in database
    assert blog.count() == 1

    # get created blog from database
    blog = blog.first()

    expected = {
        'data': {
            'attributes': {'name': blog.name, 'tags': []},
            'id': '{}'.format(blog.id),
            'links': {'self': 'http://testserver/blogs/{}'.format(blog.id)},
            'meta': {'copyright': datetime.now().year},
            'type': 'blogs'
        },
        'meta': {'apiDocs': '/docs/api/blogs'}
    }

    assert resp.status_code == 201
    assert resp.json() == expected


@pytest.mark.django_db
def test_get_object_gives_correct_blog(client, blog, entry):

    url = reverse('drf-entry-blog-detail', kwargs={'entry_pk': entry.id})
    resp = client.get(url)
    expected = {
        'data': {
            'attributes': {'name': blog.name, 'tags': []},
            'id': '{}'.format(blog.id),
            'links': {'self': 'http://testserver/blogs/{}'.format(blog.id)},
            'meta': {'copyright': datetime.now().year},
            'type': 'blogs'
        },
        'meta': {'apiDocs': '/docs/api/blogs'}
    }
    got = resp.json()
    assert got == expected


@pytest.mark.django_db
def test_get_object_patches_correct_blog(client, blog, entry):

    url = reverse('drf-entry-blog-detail', kwargs={'entry_pk': entry.id})
    new_name = blog.name + " update"
    assert not new_name == blog.name

    request_data = {
        'data': {
            'attributes': {'name': new_name},
            'id': '{}'.format(blog.id),
            'links': {'self': 'http://testserver/blogs/{}'.format(blog.id)},
            'meta': {'copyright': datetime.now().year},
            'relationships': {'tags': {'data': []}},
            'type': 'blogs'
        },
        'meta': {'apiDocs': '/docs/api/blogs'}
    }

    resp = client.patch(url, data=request_data)

    assert resp.status_code == 200

    expected = {
        'data': {
            'attributes': {'name': new_name, 'tags': []},
            'id': '{}'.format(blog.id),
            'links': {'self': 'http://testserver/blogs/{}'.format(blog.id)},
            'meta': {'copyright': datetime.now().year},
            'type': 'blogs'
        },
        'meta': {'apiDocs': '/docs/api/blogs'}
    }
    got = resp.json()
    assert got == expected


@pytest.mark.django_db
def test_get_object_deletes_correct_blog(client, entry):

    url = reverse('drf-entry-blog-detail', kwargs={'entry_pk': entry.id})

    resp = client.delete(url)

    assert resp.status_code == 204


@pytest.mark.django_db
def test_get_entry_list_with_blogs(client, entry):
    url = reverse('drf-entry-suggested', kwargs={'entry_pk': entry.id})
    resp = client.get(url)

    got = resp.json()

    expected = {
        'links': {
            'first': 'http://testserver/drf-entries/1/suggested/?page%5Bnumber%5D=1',
            'last': 'http://testserver/drf-entries/1/suggested/?page%5Bnumber%5D=1',
            'next': None,
            'prev': None,
        },
        'data': [
            {
                'type': 'entries',
                'id': '1',
                'attributes': {
                    'tags': [],
                },
                'links': {
                    'self': 'http://testserver/drf-blogs/1'
                }
            }
        ],
        'meta': {
            'pagination': {
                'page': 1,
                'pages': 1,
                'count': 1
            }
        }
    }

    assert resp.status_code == 200
    assert got == expected



import json

import pytest
from django.test import override_settings
from django.utils import timezone

from rest_framework_json_api import serializers, views
from rest_framework_json_api.renderers import JSONRenderer

from example.models import Author, Blog, Comment, Entry


# serializers
class RelatedModelSerializer(serializers.ModelSerializer):
    blog = serializers.ReadOnlyField(source='entry.blog')

    class Meta:
        model = Comment
        fields = ('id', 'blog')


class DummyTestSerializer(serializers.ModelSerializer):
    '''
    This serializer is a simple compound document serializer which includes only
    a single embedded relation
    '''
    related_models = RelatedModelSerializer(
        source='comments', many=True, read_only=True)

    json_field = serializers.SerializerMethodField()

    def get_json_field(self, entry):
        return {'JsonKey': 'JsonValue'}

    class Meta:
        model = Entry
        fields = ('related_models', 'json_field')

    class JSONAPIMeta:
        included_resources = ('related_models',)


class EntryDRFSerializers(serializers.ModelSerializer):

    class Meta:
        model = Entry
        fields = ('headline', 'body_text')
        read_only_fields = ('tags',)


class CommentWithNestedFieldsSerializer(serializers.ModelSerializer):
    entry = EntryDRFSerializers()

    class Meta:
        model = Comment
        exclude = ('created_at', 'modified_at', 'author')
        # fields = ('entry', 'body', 'author',)


class AuthorWithNestedFieldsSerializer(serializers.ModelSerializer):
    comments = CommentWithNestedFieldsSerializer(many=True)

    class Meta:
        model = Author
        fields = ('name', 'email', 'comments')


# views
class DummyTestViewSet(views.ModelViewSet):
    queryset = Entry.objects.all()
    serializer_class = DummyTestSerializer


class ReadOnlyDummyTestViewSet(views.ReadOnlyModelViewSet):
    queryset = Entry.objects.all()
    serializer_class = DummyTestSerializer


class AuthorWithNestedFieldsViewSet(views.ModelViewSet):
    queryset = Author.objects.all()
    serializer_class = AuthorWithNestedFieldsSerializer
    resource_name = 'authors'


def render_dummy_test_serialized_view(view_class, instance):
    serializer = view_class.serializer_class(instance=instance)
    renderer = JSONRenderer()
    return renderer.render(
        serializer.data,
        renderer_context={'view': view_class()})


def test_simple_reverse_relation_included_renderer():
    '''
    Test renderer when a single reverse fk relation is passed.
    '''
    rendered = render_dummy_test_serialized_view(
        DummyTestViewSet, Entry())

    assert rendered


def test_simple_reverse_relation_included_read_only_viewset():
    rendered = render_dummy_test_serialized_view(
        ReadOnlyDummyTestViewSet, Entry())

    assert rendered


def test_render_format_field_names(settings):
    """Test that json field is kept untouched."""
    settings.JSON_API_FORMAT_FIELD_NAMES = 'dasherize'
    rendered = render_dummy_test_serialized_view(DummyTestViewSet, Entry())

    result = json.loads(rendered.decode())
    assert result['data']['attributes']['json-field'] == {'JsonKey': 'JsonValue'}


def test_writeonly_not_in_response():
    """Test that writeonly fields are not shown in list response"""

    class WriteonlyTestSerializer(serializers.ModelSerializer):
        '''Serializer for testing the absence of write_only fields'''
        comments = serializers.ResourceRelatedField(
            many=True,
            write_only=True,
            queryset=Comment.objects.all()
        )

        rating = serializers.IntegerField(write_only=True)

        class Meta:
            model = Entry
            fields = ('comments', 'rating')

    class WriteOnlyDummyTestViewSet(views.ReadOnlyModelViewSet):
        queryset = Entry.objects.all()
        serializer_class = WriteonlyTestSerializer

    rendered = render_dummy_test_serialized_view(WriteOnlyDummyTestViewSet, Entry())
    result = json.loads(rendered.decode())

    assert 'rating' not in result['data']['attributes']
    assert 'relationships' not in result['data']


def test_render_empty_relationship_reverse_lookup():
    """Test that empty relationships are rendered as None."""

    class EmptyRelationshipSerializer(serializers.ModelSerializer):
        class Meta:
            model = Author
            fields = ('bio', )

    class EmptyRelationshipViewSet(views.ReadOnlyModelViewSet):
        queryset = Author.objects.all()
        serializer_class = EmptyRelationshipSerializer

    rendered = render_dummy_test_serialized_view(EmptyRelationshipViewSet, Author())
    result = json.loads(rendered.decode())
    assert 'relationships' in result['data']
    assert 'bio' in result['data']['relationships']
    assert result['data']['relationships']['bio'] == {'data': None}


@pytest.mark.django_db
def test_extract_relation_instance(comment):
    serializer = RelatedModelSerializer(instance=comment)

    got = JSONRenderer.extract_relation_instance(
        field=serializer.fields['blog'], resource_instance=comment
    )
    assert got == comment.entry.blog


def test_attribute_rendering_strategy(db):
    # setting up
    blog = Blog.objects.create(name='Some Blog', tagline="It's a blog")
    entry = Entry.objects.create(
        blog=blog,
        headline='headline',
        body_text='body_text',
        pub_date=timezone.now(),
        mod_date=timezone.now(),
        n_comments=0,
        n_pingbacks=0,
        rating=3
    )

    author = Author.objects.create(name='some_author', email='some_author@example.org')
    entry.authors.add(author)

    Comment.objects.create(
        entry=entry,
        body='testing one two three',
        author=Author.objects.first()
    )

    with override_settings(
            JSON_API_SERIALIZE_NESTED_SERIALIZERS_AS_ATTRIBUTE=True):
        rendered = render_dummy_test_serialized_view(AuthorWithNestedFieldsViewSet, author)
        result = json.loads(rendered.decode())

    expected = {
        "data": {
            "type": "authors",
            "id": "1",
            "attributes": {
                "name": "some_author",
                "email": "some_author@example.org",
                "comments": [
                    {
                        "id": 1,
                        "entry": {
                            'headline': 'headline',
                            'body_text': 'body_text',
                        },
                        "body": "testing one two three"
                    }
                ]
            }
        }
    }
    assert expected == result



import random

import pytest
from django.urls import reverse
from rest_framework import status

from example.factories import ArtProjectFactory, ProjectTypeFactory

pytestmark = pytest.mark.django_db


def test_polymorphism_on_detail(single_art_project, client):
    response = client.get(reverse("project-detail", kwargs={'pk': single_art_project.pk}))
    content = response.json()
    assert content["data"]["type"] == "artProjects"


def test_polymorphism_on_detail_relations(single_company, client):
    response = client.get(reverse("company-detail", kwargs={'pk': single_company.pk}))
    content = response.json()
    assert content["data"]["relationships"]["currentProject"]["data"]["type"] == "artProjects"
    assert (
        set([rel["type"] for rel in content["data"]["relationships"]["futureProjects"]["data"]]) ==
        set(["researchProjects", "artProjects"])
    )


def test_polymorphism_on_included_relations(single_company, client):
    response = client.get(
        reverse("company-detail", kwargs={'pk': single_company.pk}) +
        '?include=current_project,future_projects,current_art_project,current_research_project')
    content = response.json()
    assert content["data"]["relationships"]["currentProject"]["data"]["type"] == "artProjects"
    assert content["data"]["relationships"]["currentArtProject"]["data"]["type"] == "artProjects"
    assert content["data"]["relationships"]["currentResearchProject"]["data"] is None
    assert (
        set([rel["type"] for rel in content["data"]["relationships"]["futureProjects"]["data"]]) ==
        set(["researchProjects", "artProjects"])
    )
    assert set([x.get('type') for x in content.get('included')]) == set([
        'artProjects', 'artProjects', 'researchProjects']), 'Detail included types are incorrect'
    # Ensure that the child fields are present.
    assert content.get('included')[0].get('attributes').get('artist') is not None
    assert content.get('included')[1].get('attributes').get('artist') is not None
    assert content.get('included')[2].get('attributes').get('supervisor') is not None


def test_polymorphism_on_polymorphic_model_detail_patch(single_art_project, client):
    url = reverse("project-detail", kwargs={'pk': single_art_project.pk})
    response = client.get(url)
    content = response.json()
    test_topic = 'test-{}'.format(random.randint(0, 999999))
    test_artist = 'test-{}'.format(random.randint(0, 999999))
    content['data']['attributes']['topic'] = test_topic
    content['data']['attributes']['artist'] = test_artist
    response = client.patch(url, data=content)
    new_content = response.json()
    assert new_content['data']['type'] == "artProjects"
    assert new_content['data']['attributes']['topic'] == test_topic
    assert new_content['data']['attributes']['artist'] == test_artist


def test_patch_on_polymorphic_model_without_including_required_field(single_art_project, client):
    url = reverse("project-detail", kwargs={'pk': single_art_project.pk})
    data = {
        'data': {
            'id': single_art_project.pk,
            'type': 'artProjects',
            'attributes': {
                'description': 'New description'
            }
        }
    }
    response = client.patch(url, data)
    assert response.status_code == status.HTTP_200_OK
    assert response.json()['data']['attributes']['description'] == 'New description'


def test_polymorphism_on_polymorphic_model_list_post(client):
    test_topic = 'New test topic {}'.format(random.randint(0, 999999))
    test_artist = 'test-{}'.format(random.randint(0, 999999))
    test_project_type = ProjectTypeFactory()
    url = reverse('project-list')
    data = {
        'data': {
            'type': 'artProjects',
            'attributes': {
                'topic': test_topic,
                'artist': test_artist
            },
            'relationships': {
                'projectType': {
                    'data': {
                        'type': 'projectTypes',
                        'id': test_project_type.pk
                    }
                }
            }
        }
    }
    response = client.post(url, data=data)
    content = response.json()
    assert content['data']['id'] is not None
    assert content['data']['type'] == "artProjects"
    assert content['data']['attributes']['topic'] == test_topic
    assert content['data']['attributes']['artist'] == test_artist
    assert content['data']['relationships']['projectType']['data']['id'] == \
        str(test_project_type.pk)


def test_polymorphism_on_polymorphic_model_w_included_serializers(client):
    test_project = ArtProjectFactory()
    query = '?include=projectType'
    url = reverse('project-list')
    response = client.get(url + query)
    content = response.json()
    assert content['data'][0]['id'] == str(test_project.pk)
    assert content['data'][0]['type'] == 'artProjects'
    assert content['data'][0]['relationships']['projectType']['data']['id'] == \
        str(test_project.project_type.pk)
    assert content['included'][0]['type'] == 'projectTypes'
    assert content['included'][0]['id'] == str(test_project.project_type.pk)


def test_polymorphic_model_without_any_instance(client):
    expected = {
        "links": {
            'first': 'http://testserver/projects?page%5Bnumber%5D=1',
            'last': 'http://testserver/projects?page%5Bnumber%5D=1',
            "next": None,
            "prev": None
        },
        "data": [],
        "meta": {
            "pagination": {
                "page": 1,
                "pages": 1,
                "count": 0
            }
        }
    }

    response = client.get(reverse('project-list'))
    assert response.status_code == 200
    content = response.json()
    assert expected == content


def test_invalid_type_on_polymorphic_model(client):
    test_topic = 'New test topic {}'.format(random.randint(0, 999999))
    test_artist = 'test-{}'.format(random.randint(0, 999999))
    url = reverse('project-list')
    data = {
        'data': {
            'type': 'invalidProjects',
            'attributes': {
                'topic': test_topic,
                'artist': test_artist
            }
        }
    }
    response = client.post(url, data=data)
    assert response.status_code == 409
    content = response.json()
    assert len(content["errors"]) == 1
    assert content["errors"][0]["status"] == "409"
    try:
        assert content["errors"][0]["detail"] == \
            "The resource object's type (invalidProjects) is not the type that constitute the " \
            "collection represented by the endpoint (one of [researchProjects, artProjects])."
    except AssertionError:
        # Available type list order isn't enforced
        assert content["errors"][0]["detail"] == \
            "The resource object's type (invalidProjects) is not the type that constitute the " \
            "collection represented by the endpoint (one of [artProjects, researchProjects])."


def test_polymorphism_relations_update(single_company, research_project_factory, client):
    response = client.get(reverse("company-detail", kwargs={'pk': single_company.pk}))
    content = response.json()
    assert content["data"]["relationships"]["currentProject"]["data"]["type"] == "artProjects"

    research_project = research_project_factory()
    content["data"]["relationships"]["currentProject"]["data"] = {
        "type": "researchProjects",
        "id": research_project.pk
    }
    response = client.patch(reverse("company-detail", kwargs={'pk': single_company.pk}),
                            data=content)
    assert response.status_code == 200
    content = response.json()
    assert content["data"]["relationships"]["currentProject"]["data"]["type"] == "researchProjects"
    assert int(content["data"]["relationships"]["currentProject"]["data"]["id"]) == \
        research_project.pk


def test_polymorphism_relations_put_405(single_company, research_project_factory, client):
    response = client.get(reverse("company-detail", kwargs={'pk': single_company.pk}))
    content = response.json()
    assert content["data"]["relationships"]["currentProject"]["data"]["type"] == "artProjects"

    research_project = research_project_factory()
    content["data"]["relationships"]["currentProject"]["data"] = {
        "type": "researchProjects",
        "id": research_project.pk
    }
    response = client.put(reverse("company-detail", kwargs={'pk': single_company.pk}),
                          data=content)
    assert response.status_code == 405


def test_invalid_type_on_polymorphic_relation(single_company, research_project_factory, client):
    response = client.get(reverse("company-detail", kwargs={'pk': single_company.pk}))
    content = response.json()
    assert content["data"]["relationships"]["currentProject"]["data"]["type"] == "artProjects"

    research_project = research_project_factory()
    content["data"]["relationships"]["currentProject"]["data"] = {
        "type": "invalidProjects",
        "id": research_project.pk
    }
    response = client.patch(reverse("company-detail", kwargs={'pk': single_company.pk}),
                            data=content)
    assert response.status_code == 409
    content = response.json()
    assert len(content["errors"]) == 1
    assert content["errors"][0]["status"] == "409"
    try:
        assert content["errors"][0]["detail"] == \
            "Incorrect relation type. Expected one of [researchProjects, artProjects], " \
            "received invalidProjects."
    except AssertionError:
        # Available type list order isn't enforced
        assert content["errors"][0]["detail"] == \
            "Incorrect relation type. Expected one of [artProjects, researchProjects], " \
            "received invalidProjects."



from __future__ import absolute_import

import pytest
from rest_framework import serializers

from rest_framework_json_api.relations import SerializerMethodResourceRelatedField

from example.models import Blog, Entry


def test_method_name_default():
    class BlogSerializer(serializers.ModelSerializer):
        one_entry = SerializerMethodResourceRelatedField(model=Entry)

        class Meta:
            model = Blog
            fields = ['one_entry']

        def get_one_entry(self, instance):
            return Entry(id=100)

    serializer = BlogSerializer(instance=Blog())
    assert serializer.data['one_entry']['id'] == '100'


def test_method_name_custom():
    class BlogSerializer(serializers.ModelSerializer):
        one_entry = SerializerMethodResourceRelatedField(
            model=Entry,
            method_name='get_custom_entry'
        )

        class Meta:
            model = Blog
            fields = ['one_entry']

        def get_custom_entry(self, instance):
            return Entry(id=100)

    serializer = BlogSerializer(instance=Blog())
    assert serializer.data['one_entry']['id'] == '100'


@pytest.mark.filterwarnings("ignore::DeprecationWarning")
def test_source():
    class BlogSerializer(serializers.ModelSerializer):
        one_entry = SerializerMethodResourceRelatedField(
            model=Entry,
            source='get_custom_entry'
        )

        class Meta:
            model = Blog
            fields = ['one_entry']

        def get_custom_entry(self, instance):
            return Entry(id=100)

    serializer = BlogSerializer(instance=Blog())
    assert serializer.data['one_entry']['id'] == '100'


@pytest.mark.filterwarnings("error::DeprecationWarning")
def test_source_is_deprecated():
    with pytest.raises(DeprecationWarning):
        SerializerMethodResourceRelatedField(model=Entry, source='get_custom_entry')



"""
Pagination fields
"""
from collections import OrderedDict

from rest_framework.pagination import LimitOffsetPagination, PageNumberPagination
from rest_framework.utils.urls import remove_query_param, replace_query_param
from rest_framework.views import Response


class JsonApiPageNumberPagination(PageNumberPagination):
    """
    A json-api compatible pagination format.
    """
    page_query_param = 'page[number]'
    page_size_query_param = 'page[size]'
    max_page_size = 100

    def build_link(self, index):
        if not index:
            return None
        url = self.request and self.request.build_absolute_uri() or ''
        return replace_query_param(url, self.page_query_param, index)

    def get_paginated_response(self, data):
        next = None
        previous = None

        if self.page.has_next():
            next = self.page.next_page_number()
        if self.page.has_previous():
            previous = self.page.previous_page_number()

        return Response({
            'results': data,
            'meta': {
                'pagination': OrderedDict([
                    ('page', self.page.number),
                    ('pages', self.page.paginator.num_pages),
                    ('count', self.page.paginator.count),
                ])
            },
            'links': OrderedDict([
                ('first', self.build_link(1)),
                ('last', self.build_link(self.page.paginator.num_pages)),
                ('next', self.build_link(next)),
                ('prev', self.build_link(previous))
            ])
        })


class JsonApiLimitOffsetPagination(LimitOffsetPagination):
    """
    A limit/offset based style. For example:

    .. code::

        http://api.example.org/accounts/?page[limit]=100
        http://api.example.org/accounts/?page[offset]=400&page[limit]=100

    """
    limit_query_param = 'page[limit]'
    offset_query_param = 'page[offset]'
    max_limit = 100

    def get_last_link(self):
        if self.count == 0:
            return None

        url = self.request.build_absolute_uri()
        url = replace_query_param(url, self.limit_query_param, self.limit)

        offset = (self.count // self.limit) * self.limit

        if offset <= 0:
            return remove_query_param(url, self.offset_query_param)

        return replace_query_param(url, self.offset_query_param, offset)

    def get_first_link(self):
        if self.count == 0:
            return None

        url = self.request.build_absolute_uri()
        return remove_query_param(url, self.offset_query_param)

    def get_paginated_response(self, data):
        return Response({
            'results': data,
            'meta': {
                'pagination': OrderedDict([
                    ('count', self.count),
                    ('limit', self.limit),
                    ('offset', self.offset),
                ])
            },
            'links': OrderedDict([
                ('first', self.get_first_link()),
                ('last', self.get_last_link()),
                ('next', self.get_next_link()),
                ('prev', self.get_previous_link())
            ])
        })



from django.contrib.auth import models as auth_models
from django.utils import encoding
from rest_framework import generics, parsers, renderers, serializers, viewsets
from rest_framework.decorators import action
from rest_framework.response import Response

from rest_framework_json_api import utils

from example.api.serializers.identity import IdentitySerializer
from example.api.serializers.post import PostSerializer


class Identity(viewsets.ModelViewSet):
    queryset = auth_models.User.objects.all().order_by('pk')
    serializer_class = IdentitySerializer

    # demonstrate sideloading data for use at app boot time
    @action(detail=False)
    def posts(self, request):
        self.resource_name = False

        identities = self.queryset
        posts = [{'id': 1, 'title': 'Test Blog Post'}]

        data = {
            encoding.force_str('identities'): IdentitySerializer(identities, many=True).data,
            encoding.force_str('posts'): PostSerializer(posts, many=True).data,
        }
        return Response(utils.format_field_names(data, format_type='camelize'))

    @action(detail=True)
    def manual_resource_name(self, request, *args, **kwargs):
        self.resource_name = 'data'
        return super(Identity, self).retrieve(request, args, kwargs)

    @action(detail=True)
    def validation(self, request, *args, **kwargs):
        raise serializers.ValidationError('Oh nohs!')


class GenericIdentity(generics.GenericAPIView):
    """
    An endpoint that uses DRF's default format so we can test that.

    GET /identities/generic
    """
    serializer_class = IdentitySerializer
    allowed_methods = ['GET']
    renderer_classes = (renderers.JSONRenderer, )
    parser_classes = (parsers.JSONParser, )

    def get_queryset(self):
        return auth_models.User.objects.all()

    def get(self, request, pk=None):
        """
        GET request
        """
        obj = self.get_object()
        return Response(IdentitySerializer(obj).data)



from .dev import *  # noqa

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

ROOT_URLCONF = 'example.urls_test'

JSON_API_FORMAT_FIELD_NAMES = 'camelize'
JSON_API_FORMAT_TYPES = 'camelize'
JSON_API_PLURALIZE_TYPES = True

REST_FRAMEWORK.update({  # noqa
    'PAGE_SIZE': 1,
})



from collections import OrderedDict

from django.db.models.fields import related
from django.utils.encoding import force_str
from rest_framework import serializers
from rest_framework.metadata import SimpleMetadata
from rest_framework.settings import api_settings
from rest_framework.utils.field_mapping import ClassLookupDict

from rest_framework_json_api.utils import format_value, get_related_resource_type


class JSONAPIMetadata(SimpleMetadata):
    """
    This is the JSON:API metadata implementation.
    It returns an ad-hoc set of information about the view.
    There are not any formalized standards for `OPTIONS` responses
    for us to base this on.
    """
    type_lookup = ClassLookupDict({
        serializers.Field: 'GenericField',
        serializers.RelatedField: 'Relationship',
        serializers.BooleanField: 'Boolean',
        serializers.NullBooleanField: 'Boolean',
        serializers.CharField: 'String',
        serializers.URLField: 'URL',
        serializers.EmailField: 'Email',
        serializers.RegexField: 'Regex',
        serializers.SlugField: 'Slug',
        serializers.IntegerField: 'Integer',
        serializers.FloatField: 'Float',
        serializers.DecimalField: 'Decimal',
        serializers.DateField: 'Date',
        serializers.DateTimeField: 'DateTime',
        serializers.TimeField: 'Time',
        serializers.ChoiceField: 'Choice',
        serializers.MultipleChoiceField: 'MultipleChoice',
        serializers.FileField: 'File',
        serializers.ImageField: 'Image',
        serializers.ListField: 'List',
        serializers.DictField: 'Dict',
        serializers.Serializer: 'Serializer',
    })

    try:
        relation_type_lookup = ClassLookupDict({
            related.ManyToManyDescriptor: 'ManyToMany',
            related.ReverseManyToOneDescriptor: 'OneToMany',
            related.ForwardManyToOneDescriptor: 'ManyToOne',
        })
    except AttributeError:
        relation_type_lookup = ClassLookupDict({
            related.ManyRelatedObjectsDescriptor: 'ManyToMany',
            related.ReverseManyRelatedObjectsDescriptor: 'ManyToMany',
            related.ForeignRelatedObjectsDescriptor: 'OneToMany',
            related.ReverseSingleRelatedObjectDescriptor: 'ManyToOne',
        })

    def determine_metadata(self, request, view):
        metadata = OrderedDict()
        metadata['name'] = view.get_view_name()
        metadata['description'] = view.get_view_description()
        metadata['renders'] = [renderer.media_type for renderer in view.renderer_classes]
        metadata['parses'] = [parser.media_type for parser in view.parser_classes]
        metadata['allowed_methods'] = view.allowed_methods
        if hasattr(view, 'get_serializer'):
            actions = self.determine_actions(request, view)
            if actions:
                metadata['actions'] = actions
        return metadata

    def get_serializer_info(self, serializer):
        """
        Given an instance of a serializer, return a dictionary of metadata
        about its fields.
        """
        if hasattr(serializer, 'child'):
            # If this is a `ListSerializer` then we want to examine the
            # underlying child serializer instance instead.
            serializer = serializer.child

        # Remove the URL field if present
        serializer.fields.pop(api_settings.URL_FIELD_NAME, None)

        return OrderedDict([
            (format_value(field_name), self.get_field_info(field))
            for field_name, field in serializer.fields.items()
        ])

    def get_field_info(self, field):
        """
        Given an instance of a serializer field, return a dictionary
        of metadata about it.
        """
        field_info = OrderedDict()
        serializer = field.parent

        if isinstance(field, serializers.ManyRelatedField):
            field_info['type'] = self.type_lookup[field.child_relation]
        else:
            field_info['type'] = self.type_lookup[field]

        try:
            serializer_model = getattr(serializer.Meta, 'model')
            field_info['relationship_type'] = self.relation_type_lookup[
                getattr(serializer_model, field.field_name)
            ]
        except KeyError:
            pass
        except AttributeError:
            pass
        else:
            field_info['relationship_resource'] = get_related_resource_type(field)

        field_info['required'] = getattr(field, 'required', False)

        attrs = [
            'read_only', 'write_only', 'label', 'help_text',
            'min_length', 'max_length',
            'min_value', 'max_value', 'initial'
        ]

        for attr in attrs:
            value = getattr(field, attr, None)
            if value is not None and value != '':
                field_info[attr] = force_str(value, strings_only=True)

        if getattr(field, 'child', None):
            field_info['child'] = self.get_field_info(field.child)
        elif getattr(field, 'fields', None):
            field_info['children'] = self.get_serializer_info(field)

        if (
            not field_info.get('read_only') and
            not field_info.get('relationship_resource') and
            hasattr(field, 'choices')
        ):
            field_info['choices'] = [
                {
                    'value': choice_value,
                    'display_name': force_str(choice_name, strings_only=True)
                }
                for choice_value, choice_name in field.choices.items()
            ]

        if hasattr(serializer, 'included_serializers') and 'relationship_resource' in field_info:
            field_info['allows_include'] = field.field_name in serializer.included_serializers

        return field_info



import re

from rest_framework.exceptions import ValidationError
from rest_framework.filters import BaseFilterBackend, OrderingFilter

from rest_framework_json_api.utils import format_value


class OrderingFilter(OrderingFilter):
    """
    A backend filter that implements http://jsonapi.org/format/#fetching-sorting and
    raises a 400 error if any sort field is invalid.

    If you prefer *not* to report 400 errors for invalid sort fields, just use
    :py:class:`rest_framework.filters.OrderingFilter` with
    :py:attr:`~rest_framework.filters.OrderingFilter.ordering_param` = "sort"

    Also applies DJA format_value() to convert (e.g. camelcase) to underscore.
    (See JSON_API_FORMAT_FIELD_NAMES in docs/usage.md)
    """
    #: override :py:attr:`rest_framework.filters.OrderingFilter.ordering_param`
    #: with JSON:API-compliant query parameter name.
    ordering_param = 'sort'

    def remove_invalid_fields(self, queryset, fields, view, request):
        """
        Extend :py:meth:`rest_framework.filters.OrderingFilter.remove_invalid_fields` to
        validate that all provided sort fields exist (as contrasted with the super's behavior
        which is to silently remove invalid fields).

        :raises ValidationError: if a sort field is invalid.
        """
        valid_fields = [
            item[0] for item in self.get_valid_fields(queryset, view,
                                                      {'request': request})
        ]
        bad_terms = [
            term for term in fields
            if format_value(term.replace(".", "__").lstrip('-'), "underscore") not in valid_fields
        ]
        if bad_terms:
            raise ValidationError('invalid sort parameter{}: {}'.format(
                ('s' if len(bad_terms) > 1 else ''), ','.join(bad_terms)))
        # this looks like it duplicates code above, but we want the ValidationError to report
        # the actual parameter supplied while we want the fields passed to the super() to
        # be correctly rewritten.
        # The leading `-` has to be stripped to prevent format_value from turning it into `_`.
        underscore_fields = []
        for item in fields:
            item_rewritten = item.replace(".", "__")
            if item_rewritten.startswith('-'):
                underscore_fields.append(
                    '-' + format_value(item_rewritten.lstrip('-'), "underscore"))
            else:
                underscore_fields.append(format_value(item_rewritten, "underscore"))

        return super(OrderingFilter, self).remove_invalid_fields(
            queryset, underscore_fields, view, request)


class QueryParameterValidationFilter(BaseFilterBackend):
    """
    A backend filter that performs strict validation of query parameters for
    JSON:API spec conformance and raises a 400 error if non-conforming usage is
    found.

    If you want to add some additional non-standard query parameters,
    override :py:attr:`query_regex` adding the new parameters. Make sure to comply with
    the rules at http://jsonapi.org/format/#query-parameters.
    """
    #: compiled regex that matches the allowed http://jsonapi.org/format/#query-parameters:
    #: `sort` and `include` stand alone; `filter`, `fields`, and `page` have []'s
    query_regex = re.compile(r'^(sort|include)$|^(?P<type>filter|fields|page)(\[[\w\.\-]+\])?$')

    def validate_query_params(self, request):
        """
        Validate that query params are in the list of valid query keywords in
        :py:attr:`query_regex`

        :raises ValidationError: if not.
        """
        # TODO: For jsonapi error object conformance, must set jsonapi errors "parameter" for
        # the ValidationError. This requires extending DRF/DJA Exceptions.
        for qp in request.query_params.keys():
            m = self.query_regex.match(qp)
            if not m:
                raise ValidationError('invalid query parameter: {}'.format(qp))
            if not m.group('type') == 'filter' and len(request.query_params.getlist(qp)) > 1:
                raise ValidationError(
                    'repeated query parameter not allowed: {}'.format(qp))

    def filter_queryset(self, request, queryset, view):
        """
        Overrides :py:meth:`BaseFilterBackend.filter_queryset` by first validating the
        query params with :py:meth:`validate_query_params`
        """
        self.validate_query_params(request)
        return queryset



from django.urls import reverse

from example.tests import TestBase


class GenericValidationTest(TestBase):
    """
    Test that a non serializer specific validation can be thrown and formatted
    """

    def setUp(self):
        super(GenericValidationTest, self).setUp()
        self.url = reverse('user-validation', kwargs={'pk': self.miles.pk})

    def test_generic_validation_error(self):
        """
        Check error formatting
        """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 400)

        expected = {
            'errors': [{
                'status': '400',
                'source': {
                    'pointer': '/data'
                },
                'detail': 'Oh nohs!',
                'code': 'invalid',
            }]
        }

        assert expected == response.json()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2017-10-11 06:31
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('example', '0003_polymorphics'),
    ]

    operations = [
        migrations.CreateModel(
            name='AuthorType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('modified_at', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=50)),
            ],
            options={
                'ordering': ('id',),
            },
        ),
        migrations.AlterModelOptions(
            name='author',
            options={'ordering': ('id',)},
        ),
        migrations.AlterModelOptions(
            name='authorbio',
            options={'ordering': ('id',)},
        ),
        migrations.AlterModelOptions(
            name='blog',
            options={'ordering': ('id',)},
        ),
        migrations.AlterModelOptions(
            name='comment',
            options={'ordering': ('id',)},
        ),
        migrations.AlterModelOptions(
            name='entry',
            options={'ordering': ('id',)},
        ),
        migrations.AlterModelOptions(
            name='taggeditem',
            options={'ordering': ('id',)},
        ),
        migrations.AlterField(
            model_name='entry',
            name='authors',
            field=models.ManyToManyField(related_name='entries', to='example.Author'),
        ),
        migrations.AddField(
            model_name='author',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='example.AuthorType'),
        ),
    ]



from collections.abc import Iterable

from django.core.exceptions import ImproperlyConfigured
from django.db.models import Model
from django.db.models.fields.related_descriptors import (
    ForwardManyToOneDescriptor,
    ManyToManyDescriptor,
    ReverseManyToOneDescriptor,
    ReverseOneToOneDescriptor
)
from django.db.models.manager import Manager
from django.db.models.query import QuerySet
from django.urls import NoReverseMatch
from django.utils.module_loading import import_string as import_class_from_dotted_path
from rest_framework import generics, viewsets
from rest_framework.exceptions import MethodNotAllowed, NotFound
from rest_framework.fields import get_attribute
from rest_framework.relations import PKOnlyObject
from rest_framework.response import Response
from rest_framework.reverse import reverse
from rest_framework.serializers import Serializer, SkipField

from rest_framework_json_api.exceptions import Conflict
from rest_framework_json_api.serializers import ResourceIdentifierObjectSerializer
from rest_framework_json_api.utils import (
    Hyperlink,
    OrderedDict,
    get_included_resources,
    get_resource_type_from_instance
)


class PreloadIncludesMixin(object):
    """
    This mixin provides a helper attributes to select or prefetch related models
    based on the include specified in the URL.

    __all__ can be used to specify a prefetch which should be done regardless of the include


    .. code:: python

        # When MyViewSet is called with ?include=author it will prefetch author and authorbio
        class MyViewSet(viewsets.ModelViewSet):
            queryset = Book.objects.all()
            prefetch_for_includes = {
                '__all__': [],
                'category.section': ['category']
            }
            select_for_includes = {
                '__all__': [],
                'author': ['author', 'author__authorbio'],
            }
    """

    def get_select_related(self, include):
        return getattr(self, 'select_for_includes', {}).get(include, None)

    def get_prefetch_related(self, include):
        return getattr(self, 'prefetch_for_includes', {}).get(include, None)

    def get_queryset(self, *args, **kwargs):
        qs = super(PreloadIncludesMixin, self).get_queryset(*args, **kwargs)

        included_resources = get_included_resources(self.request)
        for included in included_resources + ['__all__']:

            select_related = self.get_select_related(included)
            if select_related is not None:
                qs = qs.select_related(*select_related)

            prefetch_related = self.get_prefetch_related(included)
            if prefetch_related is not None:
                qs = qs.prefetch_related(*prefetch_related)

        return qs


class AutoPrefetchMixin(object):
    def get_queryset(self, *args, **kwargs):
        """ This mixin adds automatic prefetching for OneToOne and ManyToMany fields. """
        qs = super(AutoPrefetchMixin, self).get_queryset(*args, **kwargs)

        included_resources = get_included_resources(self.request)

        for included in included_resources + ['__all__']:
            # If include was not defined, trying to resolve it automatically
            included_model = None
            levels = included.split('.')
            level_model = qs.model
            for level in levels:
                if not hasattr(level_model, level):
                    break
                field = getattr(level_model, level)
                field_class = field.__class__

                is_forward_relation = (
                    issubclass(field_class, (ForwardManyToOneDescriptor, ManyToManyDescriptor))
                )
                is_reverse_relation = (
                    issubclass(field_class, (ReverseManyToOneDescriptor, ReverseOneToOneDescriptor))
                )
                if not (is_forward_relation or is_reverse_relation):
                    break

                if level == levels[-1]:
                    included_model = field
                else:

                    if issubclass(field_class, ReverseOneToOneDescriptor):
                        model_field = field.related.field
                    else:
                        model_field = field.field

                    if is_forward_relation:
                        level_model = model_field.related_model
                    else:
                        level_model = model_field.model

            if included_model is not None:
                qs = qs.prefetch_related(included.replace('.', '__'))

        return qs


class RelatedMixin(object):
    """
    This mixin handles all related entities, whose Serializers are declared in "related_serializers"
    """

    def retrieve_related(self, request, *args, **kwargs):
        serializer_kwargs = {}
        instance = self.get_related_instance()

        if hasattr(instance, 'all'):
            instance = instance.all()

        if callable(instance):
            instance = instance()

        if instance is None:
            return Response(data=None)

        if isinstance(instance, Iterable):
            serializer_kwargs['many'] = True

        serializer = self.get_serializer(instance, **serializer_kwargs)
        return Response(serializer.data)

    def get_serializer_class(self):
        parent_serializer_class = super(RelatedMixin, self).get_serializer_class()

        if 'related_field' in self.kwargs:
            field_name = self.kwargs['related_field']

            # Try get the class from related_serializers
            if hasattr(parent_serializer_class, 'related_serializers'):
                _class = parent_serializer_class.related_serializers.get(field_name, None)
                if _class is None:
                    raise NotFound

            elif hasattr(parent_serializer_class, 'included_serializers'):
                _class = parent_serializer_class.included_serializers.get(field_name, None)
                if _class is None:
                    raise NotFound

            else:
                assert False, \
                    'Either "included_serializers" or "related_serializers" should be configured'

            if not isinstance(_class, type):
                return import_class_from_dotted_path(_class)
            return _class

        return parent_serializer_class

    def get_related_field_name(self):
        return self.kwargs['related_field']

    def get_related_instance(self):
        parent_obj = self.get_object()
        parent_serializer = self.serializer_class(parent_obj)
        field_name = self.get_related_field_name()
        field = parent_serializer.fields.get(field_name, None)

        if field is not None:
            try:
                instance = field.get_attribute(parent_obj)
            except SkipField:
                instance = get_attribute(parent_obj, field.source_attrs)
            else:
                if isinstance(instance, PKOnlyObject):
                    # need whole object
                    instance = get_attribute(parent_obj, field.source_attrs)
            return instance
        else:
            try:
                return getattr(parent_obj, field_name)
            except AttributeError:
                raise NotFound


class ModelViewSet(AutoPrefetchMixin,
                   PreloadIncludesMixin,
                   RelatedMixin,
                   viewsets.ModelViewSet):
    http_method_names = ['get', 'post', 'patch', 'delete', 'head', 'options']


class ReadOnlyModelViewSet(AutoPrefetchMixin,
                           RelatedMixin,
                           viewsets.ReadOnlyModelViewSet):
    http_method_names = ['get', 'post', 'patch', 'delete', 'head', 'options']


class RelationshipView(generics.GenericAPIView):
    serializer_class = ResourceIdentifierObjectSerializer
    self_link_view_name = None
    related_link_view_name = None
    field_name_mapping = {}
    http_method_names = ['get', 'post', 'patch', 'delete', 'head', 'options']

    def get_serializer_class(self):
        if getattr(self, 'action', False) is None:
            return Serializer
        return self.serializer_class

    def __init__(self, **kwargs):
        super(RelationshipView, self).__init__(**kwargs)
        # We include this simply for dependency injection in tests.
        # We can't add it as a class attributes or it would expect an
        # implicit `self` argument to be passed.
        self.reverse = reverse

    def get_url(self, name, view_name, kwargs, request):
        """
        Given a name, view name and kwargs, return the URL that hyperlinks to the object.

        May raise a `NoReverseMatch` if the `view_name` and `lookup_field`
        attributes are not configured to correctly match the URL conf.
        """

        # Return None if the view name is not supplied
        if not view_name:
            return None

        # Return the hyperlink, or error if incorrectly configured.
        try:
            url = self.reverse(view_name, kwargs=kwargs, request=request)
        except NoReverseMatch:
            msg = (
                'Could not resolve URL for hyperlinked relationship using '
                'view name "%s". You may have failed to include the related '
                'model in your API, or incorrectly configured the '
                '`lookup_field` attribute on this field.'
            )
            raise ImproperlyConfigured(msg % view_name)

        if url is None:
            return None

        return Hyperlink(url, name)

    def get_links(self):
        return_data = OrderedDict()
        self_link = self.get_url('self', self.self_link_view_name, self.kwargs, self.request)
        related_kwargs = {self.lookup_field: self.kwargs.get(self.lookup_field)}
        related_link = self.get_url(
            'related', self.related_link_view_name, related_kwargs, self.request
        )
        if self_link:
            return_data.update({'self': self_link})
        if related_link:
            return_data.update({'related': related_link})
        return return_data

    def get(self, request, *args, **kwargs):
        related_instance = self.get_related_instance()
        serializer_instance = self._instantiate_serializer(related_instance)
        return Response(serializer_instance.data)

    def remove_relationships(self, instance_manager, field):
        field_object = getattr(instance_manager, field)

        if field_object.null:
            for obj in instance_manager.all():
                setattr(obj, field_object.name, None)
                obj.save()
        elif hasattr(instance_manager, 'clear'):
            instance_manager.clear()
        else:
            instance_manager.all().delete()

        return instance_manager

    def patch(self, request, *args, **kwargs):
        parent_obj = self.get_object()
        related_instance_or_manager = self.get_related_instance()

        if isinstance(related_instance_or_manager, Manager):
            related_model_class = related_instance_or_manager.model
            serializer = self.get_serializer(
                data=request.data, model_class=related_model_class, many=True
            )
            serializer.is_valid(raise_exception=True)

            # for to one
            if hasattr(related_instance_or_manager, "field"):
                related_instance_or_manager = self.remove_relationships(
                    instance_manager=related_instance_or_manager, field="field")
            # for to many
            else:
                related_instance_or_manager = self.remove_relationships(
                    instance_manager=related_instance_or_manager, field="target_field")

            # have to set bulk to False since data isn't saved yet
            class_name = related_instance_or_manager.__class__.__name__
            if class_name != 'ManyRelatedManager':
                related_instance_or_manager.add(*serializer.validated_data, bulk=False)
            else:
                related_instance_or_manager.add(*serializer.validated_data)
        else:
            related_model_class = related_instance_or_manager.__class__
            serializer = self.get_serializer(data=request.data, model_class=related_model_class)
            serializer.is_valid(raise_exception=True)
            setattr(parent_obj, self.get_related_field_name(), serializer.validated_data)
            parent_obj.save()
            related_instance_or_manager = self.get_related_instance()  # Refresh instance
        result_serializer = self._instantiate_serializer(related_instance_or_manager)
        return Response(result_serializer.data)

    def post(self, request, *args, **kwargs):
        related_instance_or_manager = self.get_related_instance()

        if isinstance(related_instance_or_manager, Manager):
            related_model_class = related_instance_or_manager.model
            serializer = self.get_serializer(
                data=request.data, model_class=related_model_class, many=True
            )
            serializer.is_valid(raise_exception=True)
            if frozenset(serializer.validated_data) <= frozenset(related_instance_or_manager.all()):
                return Response(status=204)
            related_instance_or_manager.add(*serializer.validated_data)
        else:
            raise MethodNotAllowed('POST')
        result_serializer = self._instantiate_serializer(related_instance_or_manager)
        return Response(result_serializer.data)

    def delete(self, request, *args, **kwargs):
        related_instance_or_manager = self.get_related_instance()

        if isinstance(related_instance_or_manager, Manager):
            related_model_class = related_instance_or_manager.model
            serializer = self.get_serializer(
                data=request.data, model_class=related_model_class, many=True
            )
            serializer.is_valid(raise_exception=True)
            objects = related_instance_or_manager.all()
            if frozenset(serializer.validated_data).isdisjoint(frozenset(objects)):
                return Response(status=204)
            try:
                related_instance_or_manager.remove(*serializer.validated_data)
            except AttributeError:
                raise Conflict(
                    'This object cannot be removed from this relationship without being '
                    'added to another'
                )
        else:
            raise MethodNotAllowed('DELETE')
        result_serializer = self._instantiate_serializer(related_instance_or_manager)
        return Response(result_serializer.data)

    def get_related_instance(self):
        try:
            return getattr(self.get_object(), self.get_related_field_name())
        except AttributeError:
            raise NotFound

    def get_related_field_name(self):
        field_name = self.kwargs['related_field']
        if field_name in self.field_name_mapping:
            return self.field_name_mapping[field_name]
        return field_name

    def _instantiate_serializer(self, instance):
        if isinstance(instance, Model) or instance is None:
            return self.get_serializer(instance=instance)
        else:
            if isinstance(instance, (QuerySet, Manager)):
                instance = instance.all()

            return self.get_serializer(instance=instance, many=True)

    def get_resource_name(self):
        if not hasattr(self, '_resource_name'):
            instance = getattr(self.get_object(), self.get_related_field_name())
            self._resource_name = get_resource_type_from_instance(instance)
        return self._resource_name

    def set_resource_name(self, value):
        self._resource_name = value

    resource_name = property(get_resource_name, set_resource_name)



from datetime import datetime

import pytest
from django.urls import reverse

pytestmark = pytest.mark.django_db


def test_top_level_meta_for_list_view(blog, client):

    expected = {
        "data": [{
            "type": "blogs",
            "id": "1",
            "attributes": {
                "name": blog.name
            },
            "links": {
                "self": 'http://testserver/blogs/1'
            },
            'relationships': {'tags': {'data': [], 'meta': {'count': 0}}},
            "meta": {
                "copyright": datetime.now().year
            },
        }],
        'links': {
            'first': 'http://testserver/blogs?page%5Bnumber%5D=1',
            'last': 'http://testserver/blogs?page%5Bnumber%5D=1',
            'next': None,
            'prev': None
        },
        'meta': {
            'pagination': {'count': 1, 'page': 1, 'pages': 1},
            'apiDocs': '/docs/api/blogs'
        }
    }

    response = client.get(reverse("blog-list"))

    assert expected == response.json()


def test_top_level_meta_for_detail_view(blog, client):

    expected = {
        "data": {
            "type": "blogs",
            "id": "1",
            "attributes": {
                "name": blog.name
            },
            'relationships': {'tags': {'data': [], 'meta': {'count': 0}}},
            "links": {
                "self": "http://testserver/blogs/1"
            },
            "meta": {
                "copyright": datetime.now().year
            },
        },
        "meta": {
            "apiDocs": "/docs/api/blogs"
        },
    }

    response = client.get(reverse("blog-detail", kwargs={'pk': blog.pk}))

    assert expected == response.json()



import json
from datetime import datetime

from django.test import RequestFactory
from django.utils import timezone
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.exceptions import NotFound
from rest_framework.request import Request
from rest_framework.response import Response
from rest_framework.reverse import reverse
from rest_framework.test import APIRequestFactory, APITestCase, force_authenticate

from rest_framework_json_api import serializers, views
from rest_framework_json_api.utils import format_resource_type

from example.factories import AuthorFactory, CommentFactory, EntryFactory
from example.models import Author, Blog, Comment, Entry
from example.serializers import AuthorBioSerializer, AuthorTypeSerializer, EntrySerializer
from example.tests import TestBase
from example.views import AuthorViewSet, BlogViewSet


class TestRelationshipView(APITestCase):
    def setUp(self):
        self.author = Author.objects.create(name='Super powerful superhero', email='i.am@lost.com')
        self.blog = Blog.objects.create(name='Some Blog', tagline="It's a blog")
        self.other_blog = Blog.objects.create(name='Other blog', tagline="It's another blog")
        self.first_entry = Entry.objects.create(
            blog=self.blog,
            headline='headline one',
            body_text='body_text two',
            pub_date=timezone.now(),
            mod_date=timezone.now(),
            n_comments=0,
            n_pingbacks=0,
            rating=3
        )
        self.second_entry = Entry.objects.create(
            blog=self.blog,
            headline='headline two',
            body_text='body_text one',
            pub_date=timezone.now(),
            mod_date=timezone.now(),
            n_comments=0,
            n_pingbacks=0,
            rating=1
        )
        self.first_comment = Comment.objects.create(
            entry=self.first_entry, body="This entry is cool", author=None
        )
        self.second_comment = Comment.objects.create(
            entry=self.second_entry,
            body="This entry is not cool",
            author=self.author
        )

    def test_get_entry_relationship_blog(self):
        url = reverse(
            'entry-relationships', kwargs={'pk': self.first_entry.id, 'related_field': 'blog'}
        )
        response = self.client.get(url)
        expected_data = {'type': format_resource_type('Blog'), 'id': str(self.first_entry.blog.id)}

        assert response.data == expected_data

    def test_get_entry_relationship_invalid_field(self):
        response = self.client.get(
            '/entries/{}/relationships/invalid_field'.format(self.first_entry.id)
        )

        assert response.status_code == 404

    def test_get_blog_relationship_entry_set(self):
        response = self.client.get('/blogs/{}/relationships/entry_set'.format(self.blog.id))
        expected_data = [{'type': format_resource_type('Entry'), 'id': str(self.first_entry.id)},
                         {'type': format_resource_type('Entry'), 'id': str(self.second_entry.id)}]

        assert response.data == expected_data

    def test_put_entry_relationship_blog_returns_405(self):
        url = '/entries/{}/relationships/blog'.format(self.first_entry.id)
        response = self.client.put(url, data={})
        assert response.status_code == 405

    def test_patch_invalid_entry_relationship_blog_returns_400(self):
        url = '/entries/{}/relationships/blog'.format(self.first_entry.id)
        response = self.client.patch(url, data={'data': {'invalid': ''}})
        assert response.status_code == 400

    def test_relationship_view_errors_format(self):
        url = '/entries/{}/relationships/blog'.format(self.first_entry.id)
        response = self.client.patch(url, data={'data': {'invalid': ''}})
        assert response.status_code == 400

        result = json.loads(response.content.decode('utf-8'))

        assert 'data' not in result
        assert 'errors' in result

    def test_get_empty_to_one_relationship(self):
        url = '/comments/{}/relationships/author'.format(self.first_entry.id)
        response = self.client.get(url)
        expected_data = None

        assert response.data == expected_data

    def test_get_to_many_relationship_self_link(self):
        url = '/authors/{}/relationships/comments'.format(self.author.id)

        response = self.client.get(url)
        expected_data = {
            'links': {'self': 'http://testserver/authors/1/relationships/comments'},
            'data': [{'id': str(self.second_comment.id), 'type': format_resource_type('Comment')}]
        }
        assert json.loads(response.content.decode('utf-8')) == expected_data

    def test_patch_to_one_relationship(self):
        url = '/entries/{}/relationships/blog'.format(self.first_entry.id)
        request_data = {
            'data': {'type': format_resource_type('Blog'), 'id': str(self.other_blog.id)}
        }
        response = self.client.patch(url, data=request_data)
        assert response.status_code == 200, response.content.decode()
        assert response.data == request_data['data']

        response = self.client.get(url)
        assert response.data == request_data['data']

    def test_patch_one_to_many_relationship(self):
        url = '/blogs/{}/relationships/entry_set'.format(self.first_entry.id)
        request_data = {
            'data': [{'type': format_resource_type('Entry'), 'id': str(self.first_entry.id)}, ]
        }
        response = self.client.patch(url, data=request_data)
        assert response.status_code == 200, response.content.decode()
        assert response.data == request_data['data']

        response = self.client.get(url)
        assert response.data == request_data['data']

        # retry a second time should end up with same result
        response = self.client.patch(url, data=request_data)
        assert response.status_code == 200, response.content.decode()
        assert response.data == request_data['data']

        response = self.client.get(url)
        assert response.data == request_data['data']

    def test_patch_one_to_many_relaitonship_with_none(self):
        url = '/blogs/{}/relationships/entry_set'.format(self.first_entry.id)
        request_data = {
            'data': None
        }
        response = self.client.patch(url, data=request_data)
        assert response.status_code == 200, response.content.decode()
        assert response.data == []

        response = self.client.get(url)
        assert response.data == []

    def test_patch_many_to_many_relationship(self):
        url = '/entries/{}/relationships/authors'.format(self.first_entry.id)
        request_data = {
            'data': [
                {
                    'type': format_resource_type('Author'),
                    'id': str(self.author.id)
                },
            ]
        }
        response = self.client.patch(url, data=request_data)
        assert response.status_code == 200, response.content.decode()
        assert response.data == request_data['data']

        response = self.client.get(url)
        assert response.data == request_data['data']

        # retry a second time should end up with same result
        response = self.client.patch(url, data=request_data)
        assert response.status_code == 200, response.content.decode()
        assert response.data == request_data['data']

        response = self.client.get(url)
        assert response.data == request_data['data']

    def test_post_to_one_relationship_should_fail(self):
        url = '/entries/{}/relationships/blog'.format(self.first_entry.id)
        request_data = {
            'data': {'type': format_resource_type('Blog'), 'id': str(self.other_blog.id)}
        }
        response = self.client.post(url, data=request_data)
        assert response.status_code == 405, response.content.decode()

    def test_post_to_many_relationship_with_no_change(self):
        url = '/entries/{}/relationships/comments'.format(self.first_entry.id)
        request_data = {
            'data': [{'type': format_resource_type('Comment'), 'id': str(self.first_comment.id)}, ]
        }
        response = self.client.post(url, data=request_data)
        assert response.status_code == 204, response.content.decode()
        assert len(response.rendered_content) == 0, response.rendered_content.decode()

    def test_post_to_many_relationship_with_change(self):
        url = '/entries/{}/relationships/comments'.format(self.first_entry.id)
        request_data = {
            'data': [{'type': format_resource_type('Comment'), 'id': str(self.second_comment.id)}, ]
        }
        response = self.client.post(url, data=request_data)
        assert response.status_code == 200, response.content.decode()

        assert request_data['data'][0] in response.data

    def test_delete_to_one_relationship_should_fail(self):
        url = '/entries/{}/relationships/blog'.format(self.first_entry.id)
        request_data = {
            'data': {'type': format_resource_type('Blog'), 'id': str(self.other_blog.id)}
        }
        response = self.client.delete(url, data=request_data)
        assert response.status_code == 405, response.content.decode()

    def test_delete_relationship_overriding_with_none(self):
        url = '/comments/{}'.format(self.second_comment.id)
        request_data = {
            'data': {
                'type': 'comments',
                'id': self.second_comment.id,
                'relationships': {
                    'author': {
                        'data': None
                    }
                }
            }
        }
        response = self.client.patch(url, data=request_data)
        assert response.status_code == 200, response.content.decode()
        assert response.data['author'] is None

    def test_delete_to_many_relationship_with_no_change(self):
        url = '/entries/{}/relationships/comments'.format(self.first_entry.id)
        request_data = {
            'data': [{'type': format_resource_type('Comment'), 'id': str(self.second_comment.id)}, ]
        }
        response = self.client.delete(url, data=request_data)
        assert response.status_code == 204, response.content.decode()
        assert len(response.rendered_content) == 0, response.rendered_content.decode()

    def test_delete_one_to_many_relationship_with_not_null_constraint(self):
        url = '/entries/{}/relationships/comments'.format(self.first_entry.id)
        request_data = {
            'data': [{'type': format_resource_type('Comment'), 'id': str(self.first_comment.id)}, ]
        }
        response = self.client.delete(url, data=request_data)
        assert response.status_code == 409, response.content.decode()

    def test_delete_to_many_relationship_with_change(self):
        url = '/authors/{}/relationships/comments'.format(self.author.id)
        request_data = {
            'data': [{'type': format_resource_type('Comment'), 'id': str(self.second_comment.id)}, ]
        }
        response = self.client.delete(url, data=request_data)
        assert response.status_code == 200, response.content.decode()

    def test_new_comment_data_patch_to_many_relationship(self):
        entry = EntryFactory(blog=self.blog, authors=(self.author,))
        comment = CommentFactory(entry=entry)

        url = '/authors/{}/relationships/comments'.format(self.author.id)
        request_data = {
            'data': [{'type': format_resource_type('Comment'), 'id': str(comment.id)}, ]
        }
        previous_response = {
            'data': [
                {'type': 'comments',
                 'id': str(self.second_comment.id)
                 }
            ],
            'links': {
                'self': 'http://testserver/authors/{}/relationships/comments'.format(
                    self.author.id
                )
            }
        }

        response = self.client.get(url)
        assert response.status_code == 200
        assert response.json() == previous_response

        new_patched_response = {
            'data': [
                {'type': 'comments',
                 'id': str(comment.id)
                 }
            ],
            'links': {
                'self': 'http://testserver/authors/{}/relationships/comments'.format(
                    self.author.id
                )
            }
        }

        response = self.client.patch(url, data=request_data)
        assert response.status_code == 200
        assert response.json() == new_patched_response

        assert Comment.objects.filter(id=self.second_comment.id).exists()

    def test_options_entry_relationship_blog(self):
        url = reverse(
            'entry-relationships', kwargs={'pk': self.first_entry.id, 'related_field': 'blog'}
        )
        response = self.client.options(url)
        expected_data = {
            "data": {
                "name": "Entry Relationship",
                "description": "",
                "renders": [
                    "application/vnd.api+json",
                    "text/html"
                ],
                "parses": [
                    "application/vnd.api+json",
                    "application/x-www-form-urlencoded",
                    "multipart/form-data"
                ],
                "allowed_methods": [
                    "GET",
                    "POST",
                    "PATCH",
                    "DELETE",
                    "HEAD",
                    "OPTIONS"
                ],
                "actions": {
                    "POST": {}
                }
            }
        }
        assert response.json() == expected_data


class TestRelatedMixin(APITestCase):

    def setUp(self):
        self.author = AuthorFactory()

    def _get_view(self, kwargs):
        factory = APIRequestFactory()
        request = Request(factory.get('', content_type='application/vnd.api+json'))
        return AuthorViewSet(request=request, kwargs=kwargs)

    def test_get_related_field_name(self):
        kwargs = {'pk': self.author.id, 'related_field': 'bio'}
        view = self._get_view(kwargs)
        got = view.get_related_field_name()
        self.assertEqual(got, kwargs['related_field'])

    def test_get_related_instance_serializer_field(self):
        kwargs = {'pk': self.author.id, 'related_field': 'bio'}
        view = self._get_view(kwargs)
        got = view.get_related_instance()
        self.assertEqual(got, self.author.bio)

    def test_get_related_instance_model_field(self):
        kwargs = {'pk': self.author.id, 'related_field': 'id'}
        view = self._get_view(kwargs)
        got = view.get_related_instance()
        self.assertEqual(got, self.author.id)

    def test_get_serializer_class(self):
        kwargs = {'pk': self.author.id, 'related_field': 'bio'}
        view = self._get_view(kwargs)
        got = view.get_serializer_class()
        self.assertEqual(got, AuthorBioSerializer)

    def test_get_serializer_class_many(self):
        kwargs = {'pk': self.author.id, 'related_field': 'entries'}
        view = self._get_view(kwargs)
        got = view.get_serializer_class()
        self.assertEqual(got, EntrySerializer)

    def test_get_serializer_comes_from_included_serializers(self):
        kwargs = {'pk': self.author.id, 'related_field': 'type'}
        view = self._get_view(kwargs)
        related_serializers = view.serializer_class.related_serializers
        delattr(view.serializer_class, 'related_serializers')
        got = view.get_serializer_class()
        self.assertEqual(got, AuthorTypeSerializer)

        view.serializer_class.related_serializers = related_serializers

    def test_get_serializer_class_raises_error(self):
        kwargs = {'pk': self.author.id, 'related_field': 'unknown'}
        view = self._get_view(kwargs)
        self.assertRaises(NotFound, view.get_serializer_class)

    def test_retrieve_related_single_reverse_lookup(self):
        url = reverse('author-related', kwargs={'pk': self.author.pk, 'related_field': 'bio'})
        resp = self.client.get(url)
        expected = {
            'data': {
                'type': 'authorBios', 'id': str(self.author.bio.id),
                'relationships': {
                    'author': {'data': {'type': 'authors', 'id': str(self.author.id)}},
                    'metadata': {'data': {'id': str(self.author.bio.metadata.id),
                                          'type': 'authorBioMetadata'}}
                },
                'attributes': {
                    'body': str(self.author.bio.body)
                },
            }
        }
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.json(), expected)

    def test_retrieve_related_single(self):
        url = reverse('author-related', kwargs={'pk': self.author.type.pk, 'related_field': 'type'})
        resp = self.client.get(url)
        expected = {
            'data': {
                'type': 'authorTypes', 'id': str(self.author.type.id),
                'attributes': {
                    'name': str(self.author.type.name)
                },
            }
        }
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.json(), expected)

    def test_retrieve_related_many(self):
        entry = EntryFactory(authors=self.author)
        url = reverse('author-related', kwargs={'pk': self.author.pk, 'related_field': 'entries'})
        resp = self.client.get(url)

        self.assertEqual(resp.status_code, 200)
        self.assertTrue(isinstance(resp.json()['data'], list))
        self.assertEqual(len(resp.json()['data']), 1)
        self.assertEqual(resp.json()['data'][0]['id'], str(entry.id))

    def test_retrieve_related_many_hyperlinked(self):
        comment = CommentFactory(author=self.author)
        url = reverse('author-related', kwargs={'pk': self.author.pk, 'related_field': 'comments'})
        resp = self.client.get(url)

        self.assertEqual(resp.status_code, 200)
        self.assertTrue(isinstance(resp.json()['data'], list))
        self.assertEqual(len(resp.json()['data']), 1)
        self.assertEqual(resp.json()['data'][0]['id'], str(comment.id))

    def test_retrieve_related_None(self):
        kwargs = {'pk': self.author.pk, 'related_field': 'first_entry'}
        url = reverse('author-related', kwargs=kwargs)
        resp = self.client.get(url)

        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.json(), {'data': None})


class TestValidationErrorResponses(TestBase):
    def test_if_returns_error_on_empty_post(self):
        view = BlogViewSet.as_view({'post': 'create'})
        response = self._get_create_response("{}", view)
        self.assertEqual(400, response.status_code)
        expected = [{
            'detail': 'Received document does not contain primary data',
            'status': '400',
            'source': {'pointer': '/data'},
            'code': 'parse_error',
        }]
        self.assertEqual(expected, response.data)

    def test_if_returns_error_on_missing_form_data_post(self):
        view = BlogViewSet.as_view({'post': 'create'})
        response = self._get_create_response('{"data":{"attributes":{},"type":"blogs"}}', view)
        self.assertEqual(400, response.status_code)
        expected = [{
            'status': '400',
            'detail': 'This field is required.',
            'source': {'pointer': '/data/attributes/name'},
            'code': 'required',
        }]
        self.assertEqual(expected, response.data)

    def test_if_returns_error_on_bad_endpoint_name(self):
        view = BlogViewSet.as_view({'post': 'create'})
        response = self._get_create_response('{"data":{"attributes":{},"type":"bad"}}', view)
        self.assertEqual(409, response.status_code)
        expected = [{
            'detail': (
                "The resource object's type (bad) is not the type that constitute the collection "
                "represented by the endpoint (blogs)."
            ),
            'source': {'pointer': '/data'},
            'status': '409',
            'code': 'error',
        }]
        self.assertEqual(expected, response.data)

    def _get_create_response(self, data, view):
        factory = RequestFactory()
        request = factory.post('/', data, content_type='application/vnd.api+json')
        user = self.create_user('user', 'pass')
        force_authenticate(request, user)
        return view(request)


class TestModelViewSet(TestBase):
    def setUp(self):
        self.author = Author.objects.create(name='Super powerful superhero', email='i.am@lost.com')
        self.blog = Blog.objects.create(name='Some Blog', tagline="It's a blog")

    def test_no_content_response(self):
        url = '/blogs/{}'.format(self.blog.pk)
        response = self.client.delete(url)
        assert response.status_code == 204, response.rendered_content.decode()
        assert len(response.rendered_content) == 0, response.rendered_content.decode()


class TestBlogViewSet(APITestCase):

    def setUp(self):
        self.blog = Blog.objects.create(
            name='Some Blog',
            tagline="It's a blog"
        )
        self.entry = Entry.objects.create(
            blog=self.blog,
            headline='headline one',
            body_text='body_text two',
        )

    def test_get_object_gives_correct_blog(self):
        url = reverse('entry-blog', kwargs={'entry_pk': self.entry.id})
        resp = self.client.get(url)
        expected = {
            'data': {
                'attributes': {'name': self.blog.name},
                'id': '{}'.format(self.blog.id),
                'links': {'self': 'http://testserver/blogs/{}'.format(self.blog.id)},
                'meta': {'copyright': datetime.now().year},
                'relationships': {'tags': {'data': [], 'meta': {'count': 0}}},
                'type': 'blogs'
            },
            'meta': {'apiDocs': '/docs/api/blogs'}
        }
        got = resp.json()
        self.assertEqual(got, expected)


class TestEntryViewSet(APITestCase):

    def setUp(self):
        self.blog = Blog.objects.create(
            name='Some Blog',
            tagline="It's a blog"
        )
        self.first_entry = Entry.objects.create(
            blog=self.blog,
            headline='headline two',
            body_text='body_text two',
        )
        self.second_entry = Entry.objects.create(
            blog=self.blog,
            headline='headline two',
            body_text='body_text two',
        )
        self.maxDiff = None

    def test_get_object_gives_correct_entry(self):
        url = reverse('entry-featured', kwargs={'entry_pk': self.first_entry.id})
        resp = self.client.get(url)
        expected = {
            'data': {
                'attributes': {
                    'bodyText': self.second_entry.body_text,
                    'headline': self.second_entry.headline,
                    'modDate': self.second_entry.mod_date,
                    'pubDate': self.second_entry.pub_date
                },
                'id': '{}'.format(self.second_entry.id),
                'meta': {'bodyFormat': 'text'},
                'relationships': {
                    'authors': {'data': [], 'meta': {'count': 0}},
                    'blog': {
                        'data': {
                            'id': '{}'.format(self.second_entry.blog_id),
                            'type': 'blogs'
                        }
                    },
                    'blogHyperlinked': {
                        'links': {
                            'related': 'http://testserver/entries/{}'
                                       '/blog'.format(self.second_entry.id),
                            'self': 'http://testserver/entries/{}'
                                    '/relationships/blog_hyperlinked'.format(self.second_entry.id)
                        }
                    },
                    'comments': {
                        'data': [],
                        'meta': {'count': 0}
                    },
                    'commentsHyperlinked': {
                        'links': {
                            'related': 'http://testserver/entries/{}'
                                       '/comments'.format(self.second_entry.id),
                            'self': 'http://testserver/entries/{}/relationships'
                                    '/comments_hyperlinked'.format(self.second_entry.id)
                        }
                    },
                    'featuredHyperlinked': {
                        'links': {
                            'related': 'http://testserver/entries/{}'
                                       '/featured'.format(self.second_entry.id),
                            'self': 'http://testserver/entries/{}/relationships'
                                    '/featured_hyperlinked'.format(self.second_entry.id)
                        }
                    },
                    'suggested': {
                        'data': [{'id': '1', 'type': 'entries'}],
                        'links': {
                            'related': 'http://testserver/entries/{}'
                                       '/suggested/'.format(self.second_entry.id),
                            'self': 'http://testserver/entries/{}'
                                    '/relationships/suggested'.format(self.second_entry.id)
                        }
                    },
                    'suggestedHyperlinked': {
                        'links': {
                            'related': 'http://testserver/entries/{}'
                                       '/suggested/'.format(self.second_entry.id),
                            'self': 'http://testserver/entries/{}/relationships'
                                    '/suggested_hyperlinked'.format(self.second_entry.id)
                        }
                    },
                    'tags': {'data': [], 'meta': {'count': 0}}},
                'type': 'posts'
            }
        }
        got = resp.json()
        self.assertEqual(got, expected)


class BasicAuthorSerializer(serializers.ModelSerializer):
    class Meta:
        model = Author
        fields = ('name',)


class ReadOnlyViewSetWithCustomActions(views.ReadOnlyModelViewSet):
    queryset = Author.objects.all()
    serializer_class = BasicAuthorSerializer

    @action(detail=False, methods=['get', 'post', 'patch', 'delete'])
    def group_action(self, request):
        return Response(status=status.HTTP_204_NO_CONTENT)

    @action(detail=True, methods=['get', 'post', 'patch', 'delete'])
    def item_action(self, request, pk):
        return Response(status=status.HTTP_204_NO_CONTENT)


class TestReadonlyModelViewSet(TestBase):
    """
        Test if ReadOnlyModelViewSet allows to have custom actions with POST, PATCH, DELETE methods
    """
    factory = RequestFactory()
    viewset_class = ReadOnlyViewSetWithCustomActions
    media_type = 'application/vnd.api+json'

    def test_group_action_allows_get(self):
        view = self.viewset_class.as_view({'get': 'group_action'})
        request = self.factory.get('/')
        response = view(request)
        self.assertEqual(status.HTTP_204_NO_CONTENT, response.status_code)

    def test_group_action_allows_post(self):
        view = self.viewset_class.as_view({'post': 'group_action'})
        request = self.factory.post('/', '{}', content_type=self.media_type)
        response = view(request)
        self.assertEqual(status.HTTP_204_NO_CONTENT, response.status_code)

    def test_group_action_allows_patch(self):
        view = self.viewset_class.as_view({'patch': 'group_action'})
        request = self.factory.patch('/', '{}', content_type=self.media_type)
        response = view(request)
        self.assertEqual(status.HTTP_204_NO_CONTENT, response.status_code)

    def test_group_action_allows_delete(self):
        view = self.viewset_class.as_view({'delete': 'group_action'})
        request = self.factory.delete('/', '{}', content_type=self.media_type)
        response = view(request)
        self.assertEqual(status.HTTP_204_NO_CONTENT, response.status_code)

    def test_item_action_allows_get(self):
        view = self.viewset_class.as_view({'get': 'item_action'})
        request = self.factory.get('/')
        response = view(request, pk='1')
        self.assertEqual(status.HTTP_204_NO_CONTENT, response.status_code)

    def test_item_action_allows_post(self):
        view = self.viewset_class.as_view({'post': 'item_action'})
        request = self.factory.post('/', '{}', content_type=self.media_type)
        response = view(request, pk='1')
        self.assertEqual(status.HTTP_204_NO_CONTENT, response.status_code)

    def test_item_action_allows_patch(self):
        view = self.viewset_class.as_view({'patch': 'item_action'})
        request = self.factory.patch('/', '{}', content_type=self.media_type)
        response = view(request, pk='1')
        self.assertEqual(status.HTTP_204_NO_CONTENT, response.status_code)

    def test_item_action_allows_delete(self):
        view = self.viewset_class.as_view({'delete': 'item_action'})
        request = self.factory.delete('/', '{}', content_type=self.media_type)
        response = view(request, pk='1')
        self.assertEqual(status.HTTP_204_NO_CONTENT, response.status_code)
